import unittest
import time
import os
from commontest import abs_test_dir, Myrm, InternalBackup, old_test_dir, abs_output_dir
from rdiff_backup import Globals, statistics, rpath, restore


class StatsObjTest(unittest.TestCase):
    """Test StatsObj class"""

    def set_obj(self, s):
        """Set values of s's statistics"""
        s.SourceFiles = 1
        s.SourceFileSize = 2
        s.MirrorFiles = 13
        s.MirrorFileSize = 14
        s.NewFiles = 3
        s.NewFileSize = 4
        s.DeletedFiles = 5
        s.DeletedFileSize = 6
        s.ChangedFiles = 7
        s.ChangedSourceSize = 8
        s.ChangedMirrorSize = 9
        s.IncrementFiles = 15
        s.IncrementFileSize = 10
        s.StartTime = 11
        s.EndTime = 12

    def test_get_stats(self):
        """Test reading and writing stat objects"""
        s = statistics.StatsObj()
        assert s.get_stat('SourceFiles') is None
        self.set_obj(s)
        assert s.get_stat('SourceFiles') == 1

        s1 = statistics.StatFileObj()
        assert s1.get_stat('SourceFiles') == 0

    def test_get_stats_string(self):
        """Test conversion of stat object into string"""
        s = statistics.StatsObj()
        stats_string = s.get_stats_string()
        assert stats_string == "", stats_string

        self.set_obj(s)
        stats_string = s.get_stats_string()
        ss_list = stats_string.split("\n")
        tail = "\n".join(ss_list[2:])  # Time varies by time zone, don't check
        # """StartTime 11.00 (Wed Dec 31 16:00:11 1969)
        # EndTime 12.00 (Wed Dec 31 16:00:12 1969)"
        assert tail == """ElapsedTime 1.00 (1 second)
SourceFiles 1
SourceFileSize 2 (2 bytes)
MirrorFiles 13
MirrorFileSize 14 (14 bytes)
NewFiles 3
NewFileSize 4 (4 bytes)
DeletedFiles 5
DeletedFileSize 6 (6 bytes)
ChangedFiles 7
ChangedSourceSize 8 (8 bytes)
ChangedMirrorSize 9 (9 bytes)
IncrementFiles 15
IncrementFileSize 10 (10 bytes)
TotalDestinationSizeChange 7 (7 bytes)
""", "'%s'" % stats_string

    def test_line_string(self):
        """Test conversion to a single line"""
        s = statistics.StatsObj()
        self.set_obj(s)
        statline = s.get_stats_line(("sample", "index", "w", "new\nline"))
        assert statline == "sample/index/w/new\\nline 1 2 13 14 " \
            "3 4 5 6 7 8 9 15 10", repr(statline)

        statline = s.get_stats_line(())
        assert statline == ". 1 2 13 14 3 4 5 6 7 8 9 15 10"

        statline = s.get_stats_line(("file name with spaces", ))
        assert statline == "file\\x20name\\x20with\\x20spaces 1 2 13 14 " \
            "3 4 5 6 7 8 9 15 10", repr(statline)

    def test_byte_summary(self):
        """Test conversion of bytes to strings like 7.23MB"""
        s = statistics.StatsObj()
        f = s.get_byte_summary_string
        assert f(1) == "1 byte"
        assert f(234.34) == "234 bytes"
        assert f(2048) == "2.00 KB"
        assert f(3502243) == "3.34 MB"
        assert f(314992230) == "300 MB"
        assert f(36874871216) == "34.3 GB", f(36874871216)
        assert f(3775986812573450) == "3434 TB"

    def test_init_stats(self):
        """Test setting stat object from string"""
        s = statistics.StatsObj()
        s.set_stats_from_string("NewFiles 3 hello there")
        for attr in s.stat_attrs:
            if attr == 'NewFiles':
                assert s.get_stat(attr) == 3
            else:
                assert s.get_stat(attr) is None, (attr, s.__dict__[attr])

        s1 = statistics.StatsObj()
        self.set_obj(s1)
        assert not s1.stats_equal(s)

        s2 = statistics.StatsObj()
        s2.set_stats_from_string(s1.get_stats_string())
        assert s1.stats_equal(s2)

    def test_write_rp(self):
        """Test reading and writing of statistics object"""
        rp = rpath.RPath(Globals.local_connection,
                         os.path.join(abs_test_dir, b"statstest"))
        if rp.lstat():
            rp.delete()
        s = statistics.StatsObj()
        self.set_obj(s)
        s.write_stats_to_rp(rp)

        s2 = statistics.StatsObj()
        assert not s2.stats_equal(s)
        s2.read_stats_from_rp(rp)
        assert s2.stats_equal(s)

    def testAverage(self):
        """Test making an average statsobj"""
        s1 = statistics.StatsObj()
        s1.StartTime = 5
        s1.EndTime = 10
        s1.ElapsedTime = 5
        s1.ChangedFiles = 2
        s1.SourceFiles = 100
        s1.NewFileSize = 4

        s2 = statistics.StatsObj()
        s2.StartTime = 25
        s2.EndTime = 35
        s2.ElapsedTime = 10
        s2.ChangedFiles = 1
        s2.SourceFiles = 50
        s2.DeletedFiles = 0

        s3 = statistics.StatsObj().set_to_average([s1, s2])
        assert s3.StartTime is s3.EndTime is None
        assert s3.ElapsedTime == 7.5
        assert s3.DeletedFiles is s3.NewFileSize is None, (s3.DeletedFiles,
                                                           s3.NewFileSize)
        assert s3.ChangedFiles == 1.5
        assert s3.SourceFiles == 75


class IncStatTest(unittest.TestCase):
    """Test statistics as produced by actual backup"""

    def stats_check_initial(self, s):
        """Make sure stats object s compatible with initial mirroring

        A lot of the off by one stuff is because the root directory
        exists in the below examples.

        """
        assert s.MirrorFiles == 1 or s.MirrorFiles == 0
        assert s.MirrorFileSize < 20000
        assert s.NewFiles <= s.SourceFiles <= s.NewFiles + 1
        assert s.NewFileSize <= s.SourceFileSize <= s.NewFileSize + 20000
        assert s.ChangedFiles == 1 or s.ChangedFiles == 0
        assert s.ChangedSourceSize < 20000
        assert s.ChangedMirrorSize < 20000
        assert s.DeletedFiles == s.DeletedFileSize == 0
        assert s.IncrementFileSize == 0

    def testStatistics(self):
        """Test the writing of statistics

        The file sizes are approximate because the size of directories
        could change with different file systems...

        """

        def sorti(inclist):
            templist = [(inc.getinctime(), inc) for inc in inclist]
            templist.sort()
            return [inc for (t, inc) in templist]

        Globals.compression = 1
        Myrm(abs_output_dir)
        InternalBackup(1, 1, os.path.join(old_test_dir, b"stattest1"),
                       abs_output_dir)
        InternalBackup(1, 1, os.path.join(old_test_dir, b"stattest2"),
                       abs_output_dir,
                       time.time() + 1)

        rbdir = rpath.RPath(Globals.local_connection,
                            os.path.join(abs_output_dir, b"rdiff-backup-data"))

        incs = sorti(restore.get_inclist(rbdir.append("session_statistics")))
        assert len(incs) == 2
        s2 = statistics.StatsObj().read_stats_from_rp(incs[0])
        assert s2.SourceFiles == 7
        assert 700000 <= s2.SourceFileSize < 750000, s2.SourceFileSize
        self.stats_check_initial(s2)

        root_stats = statistics.StatsObj().read_stats_from_rp(incs[1])
        assert root_stats.SourceFiles == 7, root_stats.SourceFiles
        assert 550000 <= root_stats.SourceFileSize < 570000
        assert root_stats.MirrorFiles == 7
        assert 700000 <= root_stats.MirrorFileSize < 750000
        assert root_stats.NewFiles == 1
        assert root_stats.NewFileSize == 0
        assert root_stats.DeletedFiles == 1, root_stats.DeletedFiles
        assert root_stats.DeletedFileSize == 200000
        assert 3 <= root_stats.ChangedFiles <= 4, root_stats.ChangedFiles
        assert 450000 <= root_stats.ChangedSourceSize < 470000
        assert 400000 <= root_stats.ChangedMirrorSize < 420000, \
            root_stats.ChangedMirrorSize
        assert 10 < root_stats.IncrementFileSize < 30000


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import io
import time
from commontest import old_test_dir, abs_output_dir, iter_equal
from rdiff_backup import rpath, Globals, selection
from rdiff_backup.metadata import MetadataFile, PatchDiffMan, \
    quote_path, unquote_path, RORP2Record, Record2RORP, RorpExtractor

tempdir = rpath.RPath(Globals.local_connection, abs_output_dir)


class MetadataTest(unittest.TestCase):
    def make_temp(self):
        """Make temp directory testfiles/output"""
        global tempdir
        if tempdir.lstat():
            tempdir.delete()
        tempdir.mkdir()

    def testQuote(self):
        """Test quoting and unquoting"""
        filenames = [
            b"foo", b".", b"hello\nthere", b"\\", b"\\\\\\", b"h\no\t\x87\n",
            b" "
        ]
        for filename in filenames:
            quoted = quote_path(filename)
            assert b"\n" not in quoted, quoted
            result = unquote_path(quoted)
            assert result == filename, (quoted, result, filename)

    def get_rpaths(self):
        """Return list of rorps"""
        vft = rpath.RPath(Globals.local_connection,
                          os.path.join(old_test_dir, b"various_file_types"))
        rpaths = [vft.append(x) for x in vft.listdir()]
        extra_rpaths = [
            rpath.RPath(Globals.local_connection, x)
            for x in [b'/bin/ls', b'/dev/ttyS0', b'/dev/hda', b'aoeuaou']
        ]
        return [vft] + rpaths + extra_rpaths

    def testRORP2Record(self):
        """Test turning RORPs into records and back again"""
        for rp in self.get_rpaths():
            record = RORP2Record(rp)
            new_rorp = Record2RORP(record)
            assert new_rorp == rp, (new_rorp, rp, record)

    def testIterator(self):
        """Test writing RORPs to file and iterating them back"""

        def write_rorp_iter_to_file(rorp_iter, file):
            for rorp in rorp_iter:
                file.write(RORP2Record(rorp))

        rplist = self.get_rpaths()
        fp = io.BytesIO()
        write_rorp_iter_to_file(iter(rplist), fp)
        fp.seek(0)
        fp.read()
        fp.seek(0)
        outlist = list(RorpExtractor(fp).iterate())
        assert len(rplist) == len(outlist), (len(rplist), len(outlist))
        for i in range(len(rplist)):
            if not rplist[i].equal_verbose(outlist[i]):
                assert 0, (i, str(rplist[i]), str(outlist[i]))
        fp.close()

    def write_metadata_to_temp(self):
        """If necessary, write metadata of bigdir to file metadata.gz"""
        global tempdir
        temprp = tempdir.append(
            "mirror_metadata.2005-11-03T14:51:06-06:00.snapshot.gz")
        if temprp.lstat():
            return temprp

        self.make_temp()
        rootrp = rpath.RPath(Globals.local_connection,
                             os.path.join(old_test_dir, b"bigdir"))
        rpath_iter = selection.Select(rootrp).set_iter()

        start_time = time.time()
        mf = MetadataFile(temprp, 'w')
        for rp in rpath_iter:
            mf.write_object(rp)
        mf.close()
        print("Writing metadata took %s seconds" % (time.time() - start_time))
        return temprp

    def testSpeed(self):
        """Test testIterator on 10000 files"""
        temprp = self.write_metadata_to_temp()
        mf = MetadataFile(temprp, 'r')

        start_time = time.time()
        i = 0
        for rorp in mf.get_objects():
            i += 1
        print("Reading %s metadata entries took %s seconds." %
              (i, time.time() - start_time))

        start_time = time.time()
        blocksize = 32 * 1024
        with temprp.open("rb", compress=1) as tempfp:
            while 1:
                buf = tempfp.read(blocksize)
                if not buf:
                    break
        print("Simply decompressing metadata file took %s seconds" %
              (time.time() - start_time))

    def testIterate_restricted(self):
        """Test getting rorps restricted to certain index

        In this case, get assume subdir (subdir3, subdir10) has 50
        files in it.

        """
        temprp = self.write_metadata_to_temp()
        mf = MetadataFile(temprp, 'rb')
        start_time = time.time()
        i = 0
        for rorp in mf.get_objects((b"subdir3", b"subdir10")):
            i += 1
        print("Reading %s metadata entries took %s seconds." %
              (i, time.time() - start_time))
        assert i == 51

    def test_write(self):
        """Test writing to metadata file, then reading back contents"""
        global tempdir
        temprp = tempdir.append(
            "mirror_metadata.2005-11-03T12:51:06-06:00.snapshot.gz")
        if temprp.lstat():
            temprp.delete()

        self.make_temp()
        rootrp = rpath.RPath(Globals.local_connection,
                             os.path.join(old_test_dir, b"various_file_types"))
        # the following 3 lines make sure that we ignore incorrect files
        sel = selection.Select(rootrp)
        sel.ParseArgs((), ())
        rps = list(sel.set_iter())

        assert not temprp.lstat()
        write_mf = MetadataFile(temprp, 'w')
        for rp in rps:
            write_mf.write_object(rp)
        write_mf.close()
        assert temprp.lstat()

        reread_rps = list(MetadataFile(temprp, 'r').get_objects())
        assert len(reread_rps) == len(rps), (len(reread_rps), len(rps))
        for i in range(len(reread_rps)):
            assert reread_rps[i] == rps[i], i

    def test_patch(self):
        """Test combining 3 iters of metadata rorps"""
        self.make_temp()
        # shutil.copytree fails on the fifo file in the directory
        os.system(
            b'cp -a %s/* %s' %
            (os.path.join(old_test_dir, b"various_file_types"), tempdir.path))

        rp1 = tempdir.append('regular_file')
        rp2 = tempdir.append('subdir')
        rp3 = rp2.append('subdir_file')
        rp4 = tempdir.append('test')

        rp1new = tempdir.append('regular_file')
        rp1new.chmod(0)
        zero = rpath.RORPath(('test', ))

        current = [rp1, rp2, rp3]
        diff1 = [rp1, rp4]
        diff2 = [rp1new, rp2, zero]

        Globals.rbdir = tempdir
        output = PatchDiffMan().iterate_patched_meta(
            [iter(current), iter(diff1),
             iter(diff2)])
        out1 = next(output)
        assert out1 is rp1new, out1
        out2 = next(output)
        assert out2 is rp2, out2
        out3 = next(output)
        assert out3 is rp3, out3
        self.assertRaises(StopIteration, output.__next__)

    def test_meta_patch_cycle(self):
        """Create various metadata rorps, diff them, then compare"""

        def write_dir_to_meta(manager, rp, time):
            """Record the metadata under rp to a mirror_metadata file"""
            metawriter = man.get_meta_writer(b'snapshot', time)
            sel = selection.Select(rp)
            sel.ParseArgs((), ())  # make sure incorrect files are filtered out
            for rorp in sel.set_iter():
                metawriter.write_object(rorp)
            metawriter.close()

        def compare(man, rootrp, time):
            sel = selection.Select(rootrp)
            sel.ParseArgs((), ())  # make sure incorrect files are filtered out
            assert iter_equal(sel.set_iter(), man.get_meta_at_time(time, None))

        self.make_temp()
        Globals.rbdir = tempdir
        man = PatchDiffMan()
        inc1 = rpath.RPath(Globals.local_connection,
                           os.path.join(old_test_dir, b"increment1"))
        inc2 = rpath.RPath(Globals.local_connection,
                           os.path.join(old_test_dir, b"increment2"))
        inc3 = rpath.RPath(Globals.local_connection,
                           os.path.join(old_test_dir, b"increment3"))
        inc4 = rpath.RPath(Globals.local_connection,
                           os.path.join(old_test_dir, b"increment4"))
        write_dir_to_meta(man, inc1, 10000)
        compare(man, inc1, 10000)
        write_dir_to_meta(man, inc2, 20000)
        compare(man, inc2, 20000)
        man.ConvertMetaToDiff()
        man = PatchDiffMan()
        write_dir_to_meta(man, inc3, 30000)
        compare(man, inc3, 30000)
        man.ConvertMetaToDiff()
        man = PatchDiffMan()
        man.max_diff_chain = 3
        write_dir_to_meta(man, inc4, 40000)
        compare(man, inc4, 40000)
        man.ConvertMetaToDiff()

        man = PatchDiffMan()
        rplist = man.sorted_prefix_inclist(b'mirror_metadata')
        assert rplist[0].getinctype() == b'snapshot'
        assert rplist[0].getinctime() == 40000
        assert rplist[1].getinctype() == b'snapshot'
        assert rplist[1].getinctime() == 30000
        assert rplist[2].getinctype() == b'diff'
        assert rplist[2].getinctime() == 20000
        assert rplist[3].getinctype() == b'diff'
        assert rplist[3].getinctime() == 10000

        compare(man, inc1, 10000)
        compare(man, inc2, 20000)
        compare(man, inc3, 30000)
        compare(man, inc4, 40000)


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import re
from commontest import old_test_dir, abs_output_dir, MakeOutputDir
from rdiff_backup import Globals, rpath, increment, Time, Rdiff

lc = Globals.local_connection
Globals.change_source_perms = 1


def getrp(ending):
    return rpath.RPath(
        lc, os.path.join(old_test_dir, b"various_file_types", ending))


rf = getrp(b"regular_file")
rf2 = getrp(b"two_hardlinked_files1")
exec1 = getrp(b"executable")
exec2 = getrp(b"executable2")
sig = getrp(b"regular_file.sig")
hl1, hl2 = list(
    map(getrp, [b"two_hardlinked_files1", b"two_hardlinked_files2"]))
test = getrp(b"test")
dir = getrp(b".")
sym = getrp(b"symbolic_link")
nothing = getrp(b"nothing")

target = rpath.RPath(lc, os.path.join(abs_output_dir, b"out"))
out2 = rpath.RPath(lc, os.path.join(abs_output_dir, b"out2"))
out_gz = rpath.RPath(lc, os.path.join(abs_output_dir, b"out.gz"))

Time.setcurtime(1000000000)
Time.setprevtime(999424113)
prevtimestr = b"2001-09-02T02:48:33-07:00"
t_pref = os.path.join(abs_output_dir, b"out.%s" % prevtimestr)
t_diff = os.path.join(abs_output_dir, b"out.%s.diff" % prevtimestr)

Globals.no_compression_regexp = \
    re.compile(Globals.no_compression_regexp_string, re.I)


class inctest(unittest.TestCase):
    """Test the incrementRP function"""

    def setUp(self):
        Globals.set('isbackup_writer', 1)
        MakeOutputDir()

    def check_time(self, rp):
        """Make sure that rp is an inc file, and time is Time.prevtime"""
        assert rp.isincfile(), rp
        t = rp.getinctime()
        assert t == Time.prevtime, (t, Time.prevtime)

    def testreg(self):
        """Test increment of regular files"""
        Globals.compression = None
        target.setdata()
        if target.lstat():
            target.delete()
        rpd = rpath.RPath(lc, t_diff)
        if rpd.lstat():
            rpd.delete()

        diffrp = increment.Increment(rf, exec1, target)
        assert diffrp.isreg(), diffrp
        assert diffrp.equal_verbose(exec1, check_index=0, compare_size=0)
        self.check_time(diffrp)
        assert diffrp.getinctype() == b'diff', diffrp.getinctype()
        diffrp.delete()

    def testmissing(self):
        """Test creation of missing files"""
        missing_rp = increment.Increment(rf, nothing, target)
        self.check_time(missing_rp)
        assert missing_rp.getinctype() == b'missing', missing_rp.getinctype()
        missing_rp.delete()

    def testsnapshot(self):
        """Test making of a snapshot"""
        Globals.compression = None
        snap_rp = increment.Increment(rf, sym, target)
        self.check_time(snap_rp)
        assert rpath.cmp_attribs(snap_rp, sym)
        assert rpath.cmp(snap_rp, sym)
        snap_rp.delete()

        snap_rp2 = increment.Increment(sym, rf, target)
        self.check_time(snap_rp2)
        assert snap_rp2.equal_verbose(rf, check_index=0)
        assert rpath.cmp(snap_rp2, rf)
        snap_rp2.delete()

    def testGzipsnapshot(self):
        """Test making a compressed snapshot"""
        Globals.compression = 1
        rp = increment.Increment(rf, sym, target)
        self.check_time(rp)
        assert rp.equal_verbose(sym, check_index=0, compare_size=0)
        assert rpath.cmp(rp, sym)
        rp.delete()

        rp = increment.Increment(sym, rf, target)
        self.check_time(rp)
        assert rp.equal_verbose(rf, check_index=0, compare_size=0)
        assert rpath.cmpfileobj(rp.open("rb", 1), rf.open("rb"))
        assert rp.isinccompressed()
        rp.delete()

    def testdir(self):
        """Test increment on dir"""
        rp = increment.Increment(sym, dir, target)
        self.check_time(rp)
        assert rp.lstat()
        assert target.isdir()
        assert dir.equal_verbose(rp,
                                 check_index=0,
                                 compare_size=0,
                                 compare_type=0)
        assert rp.isreg()
        rp.delete()
        target.delete()

    def testDiff(self):
        """Test making diffs"""
        Globals.compression = None
        rp = increment.Increment(rf, rf2, target)
        self.check_time(rp)
        assert rp.equal_verbose(rf2, check_index=0, compare_size=0)
        Rdiff.patch_local(rf, rp, out2)
        assert rpath.cmp(rf2, out2)
        rp.delete()
        out2.delete()

    def testGzipDiff(self):
        """Test making gzipped diffs"""
        Globals.compression = 1
        rp = increment.Increment(rf, rf2, target)
        self.check_time(rp)
        assert rp.equal_verbose(rf2, check_index=0, compare_size=0)
        Rdiff.patch_local(rf, rp, out2, delta_compressed=1)
        assert rpath.cmp(rf2, out2)
        rp.delete()
        out2.delete()

    def testGzipRegexp(self):
        """Here a .gz file shouldn't be compressed"""
        Globals.compression = 1
        rpath.copy(rf, out_gz)
        assert out_gz.lstat()

        rp = increment.Increment(rf, out_gz, target)
        self.check_time(rp)
        assert rp.equal_verbose(out_gz, check_index=0, compare_size=0)
        Rdiff.patch_local(rf, rp, out2)
        assert rpath.cmp(out_gz, out2)
        rp.delete()
        out2.delete()
        out_gz.delete()


if __name__ == '__main__':
    unittest.main()



import sys
import time
import os
from commontest import abs_output_dir, re_init_subdir, abs_test_dir, Myrm
from rdiff_backup import rpath, Globals
"""benchmark.py

When possible, use 'rdiff-backup' from the shell, which allows using
different versions of rdiff-backup by altering the PYTHONPATH.  We
just use clock time, so this isn't exact at all.

"""

output_desc = abs_output_dir
new_pythonpath = None


def run_cmd(cmd):
    """Run the given cmd, return the amount of time it took"""
    time.sleep(1)  # just to be sure to not have the infamous message
    # Fatal Error: Time of Last backup is not in the past.  This is probably caused
    # by running two backups in less than a second.  Wait a second and try again.
    if new_pythonpath:
        full_cmd = b"PYTHONPATH=%b %b" % (new_pythonpath, cmd)
    else:
        full_cmd = cmd
    print("Running command '%s'" % (full_cmd, ))
    t = time.time()
    assert not os.system(full_cmd)
    return time.time() - t


def create_many_files(dirname, s, count=1000):
    """Create many short files in the dirname directory

    There will be count files in the directory, and each file will
    contain the string s.

    """
    dir_rp = rpath.RPath(Globals.local_connection, dirname)
    if (not dir_rp.isdir()):
        dir_rp.mkdir()
    for i in range(count):
        rp = dir_rp.append(b"file_%d" % i)
        fp = rp.open("w")
        fp.write(s)
        assert not fp.close()


def create_nested(dirname, s, depth, branch_factor=10):
    """Create many short files in branching directory"""

    def write(rp):
        fp = rp.open("w")
        fp.write(s)
        assert not fp.close()

    def helper(rp, depth):
        if (not rp.isdir()):
            rp.mkdir()
        sub_rps = [rp.append("file_%d" % i) for i in range(branch_factor)]
        if depth == 1:
            list(map(write, sub_rps))
        else:
            list(map(lambda rp: helper(rp, depth - 1), sub_rps))

    re_init_subdir(abs_test_dir, b'nested_out')
    helper(rpath.RPath(Globals.local_connection, dirname), depth)


def benchmark(backup_cmd, restore_cmd, desc, update_func=None):
    """Print benchmark using backup_cmd and restore_cmd

    If update_func is given, run it and then do backup a third time.

    """
    print("Initially backing up %s: %ss" % (desc, run_cmd(backup_cmd)))
    print("Updating %s, no change: %ss" % (desc, run_cmd(backup_cmd)))

    if update_func:
        update_func()
        print("Updating %s, all changed: %ss" % (desc, run_cmd(backup_cmd)))

    re_init_subdir(abs_test_dir, b'rest_out')
    print("Restoring %s to empty dir: %ss" % (desc, run_cmd(restore_cmd)))
    print("Restoring %s to unchanged dir: %ss" % (desc, run_cmd(restore_cmd)))


def many_files():
    """Time backup and restore of 2000 files"""
    count = 2000
    manyout_dir = re_init_subdir(abs_test_dir, b'many_out')
    restout_dir = re_init_subdir(abs_test_dir, b'rest_out')
    create_many_files(manyout_dir, "a", count)
    backup_cmd = b"rdiff-backup '%b' '%b'" % (manyout_dir, output_desc)
    restore_cmd = b"rdiff-backup --force -r now '%b' '%b'" % \
        (output_desc, restout_dir)

    def update_func():
        create_many_files(manyout_dir, "e", count)

    benchmark(backup_cmd, restore_cmd, "2000 1-byte files", update_func)


def many_files_no_fsync():
    """Time backup and restore of 2000 files"""
    count = 2000
    manyout_dir = re_init_subdir(abs_test_dir, b'many_out')
    restout_dir = re_init_subdir(abs_test_dir, b'rest_out')
    create_many_files(manyout_dir, "a", count)
    backup_cmd = b"rdiff-backup --no-fsync '%b' '%b'" % (manyout_dir, output_desc)
    restore_cmd = b"rdiff-backup --no-fsync --force -r now '%b' '%b'" % \
        (output_desc, restout_dir)

    def update_func():
        create_many_files(manyout_dir, "e", count)

    benchmark(backup_cmd, restore_cmd, "2000 1-byte files", update_func)


def many_files_rsync():
    """Test rsync benchmark"""
    count = 2000
    manyout_dir = re_init_subdir(abs_test_dir, b'many_out')
    create_many_files(manyout_dir, "a", count)
    rsync_command = b"rsync -e ssh -aH --delete '%s' '%s'" % \
        (manyout_dir, output_desc)
    print("Initial rsync: %ss" % (run_cmd(rsync_command), ))
    print("rsync update: %ss" % (run_cmd(rsync_command), ))

    create_many_files(manyout_dir, "e", count)
    print("Update changed rsync: %ss" % (run_cmd(rsync_command), ))


def nested_files():
    """Time backup and restore of 10000 nested files"""
    depth = 4
    nestedout_dir = re_init_subdir(abs_test_dir, b'nested_out')
    restout_dir = re_init_subdir(abs_test_dir, b'rest_out')
    create_nested(nestedout_dir, "a", depth)
    backup_cmd = b"rdiff-backup '%b' '%b'" % (nestedout_dir, output_desc)
    restore_cmd = b"rdiff-backup --force -r now '%b' '%b'" % \
        (output_desc, restout_dir)

    def update_func():
        create_nested(nestedout_dir, "e", depth)

    benchmark(backup_cmd, restore_cmd, "10000 1-byte nested files",
              update_func)


def nested_files_rsync():
    """Test rsync on nested files"""
    depth = 4
    nestedout_dir = re_init_subdir(abs_test_dir, b'nested_out')
    create_nested(nestedout_dir, "a", depth)
    rsync_command = b"rsync -e ssh -aH --delete '%b' '%b'" % \
        (nestedout_dir, output_desc)
    print("Initial rsync: %ss" % (run_cmd(rsync_command), ))
    print("rsync update: %ss" % (run_cmd(rsync_command), ))

    create_nested(nestedout_dir, "e", depth)
    print("Update changed rsync: %ss" % (run_cmd(rsync_command), ))


if len(sys.argv) < 2 or len(sys.argv) > 3:
    print("Syntax:  benchmark.py benchmark_func [output_description]")
    print("")
    print("Where output_description defaults to '%s'." % abs_output_dir)
    print("Currently benchmark_func includes:")
    print("'many_files', 'many_files_no_fsync', 'many_files_rsync', and, 'nested_files'.")
    sys.exit(1)

if len(sys.argv) == 3:
    output_desc = os.fsencode(sys.argv[2])
    if b":" not in output_desc:  # file is local
        assert not rpath.RPath(Globals.local_connection, output_desc).lstat(), \
            "Outfile file '%a' exists, try deleting it first." % (output_desc,)
else:  # we assume we can always remove the default output directory
    Myrm(output_desc)

if 'BENCHMARKPYPATH' in os.environ:
    new_pythonpath = os.fsencode(os.environ['BENCHMARKPYPATH'])

function_name = sys.argv[1]
print("Running ", function_name)
eval(sys.argv[1])()



import os
import pickle
import sys
import unittest
import time
from commontest import old_test_dir, abs_test_dir, re_init_subdir, abs_output_dir, \
    re_init_rpath_dir
from rdiff_backup import Globals, rpath


class RPathTest(unittest.TestCase):
    lc = Globals.local_connection
    mainprefix = old_test_dir
    prefix = os.path.join(mainprefix, b"various_file_types")
    write_dir = re_init_subdir(abs_test_dir, b"rpathtests")
    rp_prefix = rpath.RPath(lc, prefix, ())


class RORPStateTest(RPathTest):
    """Test Pickling of RORPaths"""

    def testPickle(self):
        rorp = rpath.RPath(self.lc, self.prefix, ("regular_file", )).getRORPath()
        rorp.file = sys.stdin  # try to confuse pickler
        assert rorp.isreg()
        rorp2 = pickle.loads(pickle.dumps(rorp, 1))
        assert rorp2.isreg()
        assert rorp2.data == rorp.data and rorp.index == rorp2.index


class CheckTypes(RPathTest):
    """Check to see if file types are identified correctly"""

    def testExist(self):
        """Can tell if files exist"""
        assert rpath.RPath(self.lc, self.prefix, ()).lstat()
        assert not rpath.RPath(self.lc, "asuthasetuouo", ()).lstat()

    def testDir(self):
        """Directories identified correctly"""
        assert rpath.RPath(self.lc, self.prefix, ()).isdir()
        assert not rpath.RPath(self.lc, self.prefix, ("regular_file", )).isdir()

    def testSym(self):
        """Symbolic links identified"""
        assert rpath.RPath(self.lc, self.prefix, ("symbolic_link", )).issym()
        assert not rpath.RPath(self.lc, self.prefix, ()).issym()

    def testReg(self):
        """Regular files identified"""
        assert rpath.RPath(self.lc, self.prefix, ("regular_file", )).isreg()
        assert not rpath.RPath(self.lc, self.prefix, ("symbolic_link", )).isreg()

    def testFifo(self):
        """Fifo's identified"""
        assert rpath.RPath(self.lc, self.prefix, ("fifo", )).isfifo()
        assert not rpath.RPath(self.lc, self.prefix, ()).isfifo()

    @unittest.skipUnless(os.path.exists('/dev/tty2'), "Test requires /dev/tty2")
    def testCharDev(self):
        """Char special files identified"""
        assert rpath.RPath(self.lc, "/dev/tty2", ()).ischardev()
        assert not rpath.RPath(self.lc, self.prefix, ("regular_file", )).ischardev()

    @unittest.skipUnless(
        os.path.exists('/dev/sda') or os.path.exists('/dev/nvme0n1'),
        "Test requires either /dev/sda or /dev/nvme0n1")
    def testBlockDev(self):
        """Block special files identified"""
        # Introducing a new dependency just for a few tests doesn't sound
        # reasonable, especially as it doesn't solve minor/major questions
        # somediskdev = os.path.realpath(psutil.disk_partitions()[0].device)
        # We assume that anybody must have a hard drive, SSD or NVMe
        if (os.path.exists('/dev/sda')):
            assert rpath.RPath(self.lc, '/dev/sda', ()).isblkdev()
        else:
            assert rpath.RPath(self.lc, '/dev/nvme0n1', ()).isblkdev()
        assert not rpath.RPath(self.lc, self.prefix, ("regular_file", )).isblkdev()


class CheckPerms(RPathTest):
    """Check to see if permissions are reported and set accurately"""

    def testExecReport(self):
        """Check permissions for executable files"""
        assert self.rp_prefix.append('executable').getperms() == 0o755
        assert self.rp_prefix.append('executable2').getperms() == 0o700

    def testhighbits(self):
        """Test reporting of highbit permissions"""
        p = rpath.RPath(self.lc, os.path.join(self.mainprefix, b"rpath2",
                                              b"foobar")).getperms()
        assert p == 0o4100, p

    def testOrdinaryReport(self):
        """Ordinary file permissions..."""
        assert self.rp_prefix.append("regular_file").getperms() == 0o644
        assert self.rp_prefix.append(
            'two_hardlinked_files1').getperms() == 0o640

    def testChmod(self):
        """Test changing file permission"""
        rp = self.rp_prefix.append("changeable_permission")
        rp.chmod(0o700)
        assert rp.getperms() == 0o700
        rp.chmod(0o644)
        assert rp.getperms() == 0o644

    def testExceptions(self):
        """What happens when file absent"""
        self.assertRaises(
            Exception,
            rpath.RPath(self.lc, self.prefix, ("aoeunto", )).getperms())


class CheckTimes(RPathTest):
    """Check to see if times are reported and set accurately"""

    def testSet(self):
        """Check to see if times set properly"""
        rp = rpath.RPath(self.lc, self.prefix, ("timetest.foo", ))
        rp.touch()
        rp.settime(10000, 20000)
        rp.setdata()
        assert rp.getatime() == 10000
        assert rp.getmtime() == 20000
        rp.delete()

    def testCtime(self):
        """Check to see if ctime read, compared"""
        rp = rpath.RPath(self.lc, self.prefix, ("ctimetest.1", ))
        rp2 = rpath.RPath(self.lc, self.prefix, ("ctimetest.2", ))
        rp.touch()
        rp.chmod(0o700)
        rpath.copy_with_attribs(rp, rp2)
        assert rpath.cmp_attribs(rp, rp2)

        time.sleep(1)
        rp2.chmod(0o755)
        rp2.chmod(0o700)
        rp2.setdata()
        assert rp2.getctime() > rp.getctime()
        assert not rpath.cmp_attribs(rp, rp2)
        rp.delete()
        rp2.delete()


class CheckDir(RPathTest):
    """Check directory related functions"""

    def testCreation(self):
        """Test directory creation and deletion"""
        d = self.rp_prefix.append("tempdir")
        assert not d.lstat()
        d.mkdir()
        assert d.isdir()
        d.rmdir()
        assert not d.lstat()

    def testExceptions(self):
        """Should raise os.errors when no files"""
        d = rpath.RPath(self.lc, self.prefix, ("suthosutho", ))
        self.assertRaises(os.error, d.rmdir)
        d.mkdir()
        self.assertRaises(os.error, d.mkdir)
        d.rmdir()

    def testListdir(self):
        """Checking dir listings"""
        dirlist = rpath.RPath(self.lc, self.mainprefix, ("sampledir", )).listdir()
        dirlist.sort()
        assert dirlist == [b"1", b"2", b"3", b"4"], dirlist


class CheckSyms(RPathTest):
    """Check symlinking and reading"""

    def testRead(self):
        """symlink read"""
        assert (rpath.RPath(self.lc, self.prefix,
                            ("symbolic_link", )).readlink() == b"regular_file")

    def testMake(self):
        """Creating symlink"""
        link = rpath.RPath(self.lc, self.write_dir, ("symlink", ))
        assert not link.lstat()
        link.symlink("abcdefg")
        assert link.issym()
        assert link.readlink() == b"abcdefg"
        link.delete()


class CheckSockets(RPathTest):
    """Check reading and making sockets"""

    def testMake(self):
        """Create socket, then read it"""
        sock = rpath.RPath(self.lc, self.write_dir, ("socket", ))
        assert not sock.lstat()
        sock.mksock()
        assert sock.issock()
        sock.delete()

    def testLongSock(self):
        """Test making a socket with a long name

        On some systems, the name of a socket is restricted, and
        cannot be as long as a regular file.  When this happens, a
        SkipFileException should be raised.

        """
        sock = rpath.RPath(self.lc, self.write_dir, (
            "socketaoeusthaoeaoeutnhaonseuhtansoeuthasoneuthasoeutnhasonuthaoensuhtasoneuhtsanouhonetuhasoneuthsaoenaonsetuaosenuhtaoensuhaoeu",
        ))
        assert not sock.lstat()
        try:
            sock.mksock()
        except rpath.SkipFileException:
            pass
        else:
            print("Warning, making long socket did not fail")
        sock.setdata()
        if sock.lstat():
            sock.delete()


class TouchDelete(RPathTest):
    """Check touching and deletion of files"""

    def testTouch(self):
        """Creation of 0 length files"""
        t = rpath.RPath(self.lc, self.write_dir, ("testtouch", ))
        assert not t.lstat()
        t.touch()
        assert t.lstat()
        t.delete()

    def testDelete(self):
        """Deletion of files"""
        d = rpath.RPath(self.lc, self.write_dir, ("testdelete", ))
        d.touch()
        assert d.lstat()
        d.delete()
        assert not d.lstat()


class MiscFileInfo(RPathTest):
    """Check Miscellaneous file information"""

    def testFileLength(self):
        """File length = getsize()"""
        assert (rpath.RPath(self.lc, self.prefix,
                            ("regular_file", )).getsize() == 75650)


class FilenameOps(RPathTest):
    """Check filename operations"""
    weirdfilename = eval(
        '\'\\xd8\\xab\\xb1Wb\\xae\\xc5]\\x8a\\xbb\\x15v*\\xf4\\x0f!\\xf9>\\xe2Y\\x86\\xbb\\xab\\xdbp\\xb0\\x84\\x13k\\x1d\\xc2\\xf1\\xf5e\\xa5U\\x82\\x9aUV\\xa0\\xf4\\xdf4\\xba\\xfdX\\x03\\x82\\x07s\\xce\\x9e\\x8b\\xb34\\x04\\x9f\\x17 \\xf4\\x8f\\xa6\\xfa\\x97\\xab\\xd8\\xac\\xda\\x85\\xdcKvC\\xfa#\\x94\\x92\\x9e\\xc9\\xb7\\xc3_\\x0f\\x84g\\x9aB\\x11<=^\\xdbM\\x13\\x96c\\x8b\\xa7|*"\\\\\\\'^$@#!(){}?+ ~` \''
    )
    normdict = {
        b"/": b"/",
        b".": b".",
        b"//": b"/",
        b"/a/b": b"/a/b",
        b"a/b": b"a/b",
        b"a//b": b"a/b",
        b"a////b//c": b"a/b/c",
        b"..": b"..",
        b"a/": b"a",
        b"/a//b///": b"/a/b",
        b"//host/share": b"//host/share",
        b"//host//share/": b"//host/share",
    }
    dirsplitdict = {
        b"/": (b"", b""),
        b"/a": (b"", b"a"),
        b"/a/b": (b"/a", b"b"),
        b".": (b".", b"."),
        b"b/c": (b"b", b"c"),
        b"a": (b".", b"a"),
        b"//host/share": (b"//host", b"share"),
    }

    def testQuote(self):
        """See if filename quoting works"""
        wtf = rpath.RPath(self.lc, self.prefix, (self.weirdfilename, ))
        reg = rpath.RPath(self.lc, self.prefix, (b"regular_file", ))
        assert wtf.lstat()
        assert reg.lstat()
        assert not os.system(b"ls %s >/dev/null 2>&1" % wtf.quote())
        assert not os.system(b"ls %s >/dev/null 2>&1" % reg.quote())

    def testNormalize(self):
        """rpath.normalize() dictionary test"""
        for (before, after) in list(self.normdict.items()):
            assert rpath.RPath(self.lc, before, ()).normalize().path == after, \
                "Normalize fails for %a => %a" % (before, after)

    def testDirsplit(self):
        """Test splitting of various directories"""
        for full, split in list(self.dirsplitdict.items()):
            result = rpath.RPath(self.lc, full, ()).dirsplit()
            assert result == split, \
                "%s => %s instead of %s" % (full, result, split)

    @unittest.skipUnless(
        (os.path.exists('/dev/sda') or os.path.exists('/dev/nvme0n1'))
        and os.path.exists('/dev/tty2'),
        "Test requires either /dev/sda or /dev/nvme0n1")
    def testGetnums(self):
        """Test getting file numbers"""
        if (os.path.exists(b'/dev/sda')):
            devnums = rpath.RPath(self.lc, b"/dev/sda", ()).getdevnums()
            self.assertEqual(devnums, ('b', 8, 0))
        else:
            devnums = rpath.RPath(self.lc, b"/dev/nvme0n1", ()).getdevnums()
            self.assertEqual(devnums, ('b', 259, 0))
        devnums = rpath.RPath(self.lc, b"/dev/tty2", ()).getdevnums()
        self.assertEqual(devnums, ('c', 4, 2))


class FileIO(RPathTest):
    """Test file input and output"""

    def testRead(self):
        """File reading"""
        with rpath.RPath(self.lc, self.prefix, ("executable", )).open("r") as fp:
            assert fp.read(6) == "#!/bin"

    def testWrite(self):
        """File writing"""
        rp = rpath.RPath(self.lc, self.mainprefix, ("testfile", ))
        with rp.open("w") as fp:
            fp.write("hello")
        with rp.open("r") as fp_input:
            assert fp_input.read() == "hello"
        rp.delete()

    def testGzipWrite(self):
        """Test writing of gzipped files"""
        try:
            os.mkdir(abs_output_dir)
        except OSError:
            pass
        file_nogz = os.path.join(abs_output_dir, b"file")
        file_gz = file_nogz + b".gz"
        rp_gz = rpath.RPath(self.lc, file_gz)
        rp_nogz = rpath.RPath(self.lc, file_nogz)
        if rp_nogz.lstat():
            rp_nogz.delete()
        s = b"Hello, world!"

        with rp_gz.open("wb", compress=1) as fp_out:
            fp_out.write(s)
        assert not os.system(b"gunzip %s" % file_gz)
        with rp_nogz.open("rb") as fp_in:
            assert fp_in.read() == s

    def testGzipRead(self):
        """Test reading of gzipped files"""
        try:
            os.mkdir(abs_output_dir)
        except OSError:
            pass
        file_nogz = os.path.join(abs_output_dir, b"file")
        file_gz = file_nogz + b".gz"
        rp_gz = rpath.RPath(self.lc, file_gz)
        if rp_gz.lstat():
            rp_gz.delete()
        rp_nogz = rpath.RPath(self.lc, file_nogz)
        s = "Hello, world!"

        with rp_nogz.open("w") as fp_out:
            fp_out.write(s)
        rp_nogz.setdata()
        assert rp_nogz.lstat()

        assert not os.system(b"gzip %s" % file_nogz)
        rp_nogz.setdata()
        rp_gz.setdata()
        assert not rp_nogz.lstat()
        assert rp_gz.lstat()
        with rp_gz.open("r", compress=1) as fp_in:
            read_s = fp_in.read().decode()  # zip is always binary hence bytes
            assert read_s == s, "Read string %s not like written %s." % (
                read_s, s)


class FileCopying(RPathTest):
    """Test file copying and comparison"""

    def setUp(self):
        self.hl1 = rpath.RPath(self.lc, self.prefix, ("two_hardlinked_files1", ))
        self.hl2 = rpath.RPath(self.lc, self.prefix, ("two_hardlinked_files2", ))
        self.sl = rpath.RPath(self.lc, self.prefix, ("symbolic_link", ))
        self.dir = rpath.RPath(self.lc, self.prefix, ())
        self.fifo = rpath.RPath(self.lc, self.prefix, ("fifo", ))
        self.rf = rpath.RPath(self.lc, self.prefix, ("regular_file", ))
        self.dest = rpath.RPath(self.lc, self.mainprefix, ("dest", ))
        if self.dest.lstat():
            self.dest.delete()
        assert not self.dest.lstat()

    def testComp(self):
        """Test comparisons involving regular files"""
        assert rpath.cmp(self.hl1, self.hl2)
        assert not rpath.cmp(self.rf, self.hl1)
        assert not rpath.cmp(self.dir, self.rf)

    def testCompMisc(self):
        """Test miscellaneous comparisons"""
        assert rpath.cmp(self.dir, rpath.RPath(self.lc, self.mainprefix, ()))
        self.dest.symlink("regular_file")
        assert rpath.cmp(self.sl, self.dest)
        self.dest.delete()
        assert not rpath.cmp(self.sl, self.fifo)
        assert not rpath.cmp(self.dir, self.sl)

    def testDirSizeComp(self):
        """Make sure directories can be equal,
        even if they are of different sizes"""
        smalldir = rpath.RPath(Globals.local_connection,
                               os.path.join(old_test_dir, b"dircomptest", b"1"))
        bigdir = rpath.RPath(Globals.local_connection,
                             os.path.join(old_test_dir, b"dircomptest", b"2"))
        # Can guarantee below by adding files to bigdir
        assert bigdir.getsize() > smalldir.getsize()
        assert smalldir == bigdir

    def testCopy(self):
        """Test copy of various files"""
        for rp in [self.sl, self.rf, self.fifo, self.dir]:
            rpath.copy(rp, self.dest)
            assert self.dest.lstat(), "%a doesn't exist" % self.dest.path
            assert rpath.cmp(rp, self.dest)
            assert rpath.cmp(self.dest, rp)
            self.dest.delete()


class FileAttributes(FileCopying):
    """Test file attribute operations"""

    def setUp(self):
        FileCopying.setUp(self)
        self.noperms = rpath.RPath(self.lc, self.mainprefix, ("noperms", ))
        self.nowrite = rpath.RPath(self.lc, self.mainprefix, ("nowrite", ))
        self.exec1 = rpath.RPath(self.lc, self.prefix, ("executable", ))
        self.exec2 = rpath.RPath(self.lc, self.prefix, ("executable2", ))
        self.test = rpath.RPath(self.lc, self.prefix, ("test", ))
        self.nothing = rpath.RPath(self.lc, self.prefix, ("aoeunthoenuouo", ))
        self.sym = rpath.RPath(self.lc, self.prefix, ("symbolic_link", ))

    def testComp(self):
        """Test attribute comparison success"""
        testpairs = [(self.hl1, self.hl2)]
        for a, b in testpairs:
            assert a.equal_loose(b), "Err with %a %a" % (a.path, b.path)
            assert b.equal_loose(a), "Err with %a %a" % (b.path, a.path)

    def testCompFail(self):
        """Test attribute comparison failures"""
        testpairs = [(self.nowrite, self.noperms), (self.exec1, self.exec2),
                     (self.rf, self.hl1)]
        for a, b in testpairs:
            assert not a.equal_loose(b), "Err with %a %a" % (a.path, b.path)
            assert not b.equal_loose(a), "Err with %a %a" % (b.path, a.path)

    def testCheckRaise(self):
        """Should raise exception when file missing"""
        self.assertRaises(rpath.RPathException, rpath.check_for_files, self.nothing,
                          self.hl1)
        self.assertRaises(rpath.RPathException, rpath.check_for_files, self.hl1,
                          self.nothing)

    def testCopyAttribs(self):
        """Test copying attributes"""
        t = rpath.RPath(self.lc, self.write_dir, ("testattribs", ))
        if t.lstat():
            t.delete()
        for rp in [
                self.noperms, self.nowrite, self.rf, self.exec1, self.exec2,
                self.hl1, self.dir
        ]:
            rpath.copy(rp, t)
            rpath.copy_attribs(rp, t)
            assert t.equal_loose(rp), \
                "Attributes for file %s not copied successfully" % rp.path
            t.delete()

    def testCopyWithAttribs(self):
        """Test copying with attribs (bug found earlier)"""
        out = rpath.RPath(self.lc, self.write_dir, ("out", ))
        if out.lstat():
            out.delete()
        for rp in [
                self.noperms, self.nowrite, self.rf, self.exec1, self.exec2,
                self.hl1, self.dir, self.sym
        ]:
            rpath.copy_with_attribs(rp, out)
            assert rpath.cmp(rp, out)
            assert rp.equal_loose(out)
            out.delete()

    def testCopyRaise(self):
        """Should raise exception for non-existent files"""
        self.assertRaises(AssertionError, rpath.copy_attribs, self.hl1,
                          self.nothing)
        self.assertRaises(AssertionError, rpath.copy_attribs, self.nothing,
                          self.nowrite)


class CheckPath(unittest.TestCase):
    """Check to make sure paths generated properly"""

    def testpath(self):
        """Test root paths"""
        root = rpath.RPath(Globals.local_connection, "/")
        assert root.path == b"/", root.path
        bin = root.append("bin")
        assert bin.path == b"/bin", bin.path
        bin2 = rpath.RPath(Globals.local_connection, "/bin")
        assert bin2.path == b"/bin", bin2.path


class Gzip(RPathTest):
    """Test the gzip related functions/classes"""

    def test_maybe_gzip(self):
        """Test MaybeGzip"""
        dirrp = rpath.RPath(self.lc, abs_output_dir)
        re_init_rpath_dir(dirrp)

        base_rp = dirrp.append('foo')
        fileobj = rpath.MaybeGzip(base_rp)
        fileobj.close()
        base_rp.setdata()
        assert base_rp.isreg(), base_rp
        assert base_rp.getsize() == 0
        base_rp.delete()

        base_gz = dirrp.append('foo.gz')
        assert not base_gz.lstat()
        fileobj = rpath.MaybeGzip(base_rp)
        fileobj.write(b"lala")
        fileobj.close()
        base_rp.setdata()
        base_gz.setdata()
        assert not base_rp.lstat()
        assert base_gz.isreg(), base_gz
        data = base_gz.get_bytes(compressed=1)
        assert data == b"lala", data


if __name__ == "__main__":
    unittest.main()



import unittest
from rdiff_backup import SetConnections


class SetConnectionsTest(unittest.TestCase):
    """Set SetConnections Class"""

    def testParsing(self):
        """Test parsing of various file descriptors"""
        pfd = SetConnections.parse_file_desc
        self.assertEqual(pfd(b"bescoto@folly.stanford.edu::/usr/bin/ls"),
                         (b"bescoto@folly.stanford.edu", b"/usr/bin/ls"))
        self.assertEqual(pfd(b"hello there::/goodbye:euoeu"),
                         (b"hello there", b"/goodbye:euoeu"))
        self.assertEqual(pfd(rb"test\\ing\::more::and more\\.."),
                         (rb"test\ing::more", rb"and more\\.."))
        self.assertEqual(pfd(b"a:b:c:d::e"), (b"a:b:c:d", b"e"))
        self.assertEqual(pfd(b"foobar"), (None, b"foobar"))
        self.assertEqual(pfd(rb"hello\::there"), (None, rb"hello\::there"))
        self.assertEqual(pfd(rb"foobar\\"), (None, rb"foobar\\"))

        # test missing path and missing host
        self.assertRaises(SetConnections.SetConnectionsException, pfd, rb"hello\:there::")
        self.assertRaises(SetConnections.SetConnectionsException, pfd, b"::some/path/without/host")


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import io
import pwd
import grp
from rdiff_backup.eas_acls import AccessControlLists, metadata, ACLExtractor, \
    Record2ACL, ACL2Record, ExtendedAttributes, EAExtractor, EA2Record, Record2EA
from rdiff_backup import Globals, rpath, user_group
from commontest import rdiff_backup, abs_test_dir, abs_output_dir, abs_restore_dir, \
    BackupRestoreSeries, CompareRecursive

user_group.init_user_mapping()
user_group.init_group_mapping()
tempdir = rpath.RPath(Globals.local_connection, abs_output_dir)
restore_dir = rpath.RPath(Globals.local_connection, abs_restore_dir)


class EATest(unittest.TestCase):
    """Test extended attributes"""
    sample_ea = ExtendedAttributes(
        (), {
            b'user.empty':
            b'',
            b'user.not_empty':
            b'foobar',
            b'user.third':
            b'hello',
            b'user.binary':
            bytes((0, 1, 2, 140)) + b'/="',
            b'user.multiline':
            b"""This is a fairly long extended attribute.
                Encoding it will require several lines of
                base64.""" + bytes((177, ) * 300)
        })
    empty_ea = ExtendedAttributes(())
    ea1 = ExtendedAttributes(('e1', ), sample_ea.attr_dict.copy())
    ea1.delete(b'user.not_empty')
    ea2 = ExtendedAttributes(('e2', ), sample_ea.attr_dict.copy())
    ea2.set(b'user.third', b'Another random attribute')
    ea3 = ExtendedAttributes(('e3', ))
    ea4 = ExtendedAttributes(('e4', ),
                             {b'user.deleted': b'File to be deleted'})
    ea_test1_dir = os.path.join(abs_test_dir, b'ea_test1')
    ea_test1_rpath = rpath.RPath(Globals.local_connection, ea_test1_dir)
    ea_test2_dir = os.path.join(abs_test_dir, b'ea_test2')
    ea_test2_rpath = rpath.RPath(Globals.local_connection, ea_test2_dir)
    ea_empty_dir = os.path.join(abs_test_dir, b'ea_empty')
    ea_empty_rpath = rpath.RPath(Globals.local_connection, ea_empty_dir)

    def make_temp_out_dirs(self):
        """Make temp output and restore directories empty"""
        tempdir.setdata()  # in case the file changed in-between
        if tempdir.lstat():
            tempdir.delete()
        tempdir.mkdir()
        restore_dir.setdata()
        if restore_dir.lstat():
            restore_dir.delete()

    def testBasic(self):
        """Test basic writing and reading of extended attributes"""
        self.make_temp_out_dirs()
        new_ea = ExtendedAttributes(())
        new_ea.read_from_rp(tempdir)
        # we ignore SELinux extended attributes for comparaison
        if new_ea.attr_dict:
            new_ea.attr_dict.pop(b'security.selinux', None)
        assert not new_ea.attr_dict, "The attributes of %s should have been empty: %s" % (
            tempdir, new_ea.attr_dict)
        assert not new_ea == self.sample_ea
        assert new_ea != self.sample_ea
        assert new_ea == self.empty_ea

        self.sample_ea.write_to_rp(tempdir)
        new_ea.read_from_rp(tempdir)
        if new_ea.attr_dict:
            new_ea.attr_dict.pop(b'security.selinux', None)
        assert new_ea.attr_dict == self.sample_ea.attr_dict, \
            (new_ea.attr_dict, self.sample_ea.attr_dict)
        assert new_ea == self.sample_ea

    def testRecord(self):
        """Test writing a record and reading it back"""
        record = EA2Record(self.sample_ea)
        new_ea = Record2EA(record)
        if not new_ea == self.sample_ea:
            new_list = list(new_ea.attr_dict.keys())
            sample_list = list(self.sample_ea.attr_dict.keys())
            new_list.sort()
            sample_list.sort()
            assert new_list == sample_list, (new_list, sample_list)
            for name in new_list:
                assert self.sample_ea.get(name) == new_ea.get(name), \
                    (self.sample_ea.get(name), new_ea.get(name))
            assert self.sample_ea.index == new_ea.index, \
                (self.sample_ea.index, new_ea.index)
            assert 0, "We shouldn't have gotten this far"

    def testExtractor(self):
        """Test seeking inside a record list"""
        record_list = """# file: 0foo
user.multiline=0sVGhpcyBpcyBhIGZhaXJseSBsb25nIGV4dGVuZGVkIGF0dHJpYnV0ZS4KCQkJIEVuY29kaW5nIGl0IHdpbGwgcmVxdWlyZSBzZXZlcmFsIGxpbmVzIG9mCgkJCSBiYXNlNjQusbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGx
user.third=0saGVsbG8=
user.not_empty=0sZm9vYmFy
user.binary=0sAAECjC89Ig==
user.empty
# file: 1foo/bar/baz
user.multiline=0sVGhpcyBpcyBhIGZhaXJseSBsb25nIGV4dGVuZGVkIGF0dHJpYnV0ZS4KCQkJIEVuY29kaW5nIGl0IHdpbGwgcmVxdWlyZSBzZXZlcmFsIGxpbmVzIG9mCgkJCSBiYXNlNjQusbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGxsbGx
user.third=0saGVsbG8=
user.binary=0sAAECjC89Ig==
user.empty
# file: 2foo/\\012
user.empty
"""
        extractor = EAExtractor(io.BytesIO(os.fsencode(record_list)))
        ea_iter = extractor.iterate_starting_with(())
        first = next(ea_iter)
        assert first.index == (b'0foo', ), first
        second = next(ea_iter)
        assert second.index == (b'1foo', b'bar', b'baz'), second
        third = next(ea_iter)  # Test quoted filenames
        assert third.index == (b'2foo', b'\n'), third.index
        try:
            next(ea_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Too many elements in iterator"

        extractor = EAExtractor(io.BytesIO(os.fsencode(record_list)))
        ea_iter = extractor.iterate_starting_with((b'1foo', b'bar'))
        assert next(ea_iter).index == (b'1foo', b'bar', b'baz')
        try:
            next(ea_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Too many elements in iterator"

    def make_backup_dirs(self):
        """Create testfiles/ea_test[12] directories

        Goal is to set range of extended attributes, to give good test
        to extended attribute code.

        """
        if self.ea_test1_rpath.lstat():
            self.ea_test1_rpath.delete()
        if self.ea_test2_rpath.lstat():
            self.ea_test2_rpath.delete()
        self.ea_test1_rpath.mkdir()
        rp1_1 = self.ea_test1_rpath.append('e1')
        rp1_2 = self.ea_test1_rpath.append('e2')
        rp1_3 = self.ea_test1_rpath.append('e3')
        rp1_4 = self.ea_test1_rpath.append('e4')
        list(map(rpath.RPath.touch, [rp1_1, rp1_2, rp1_3, rp1_4]))
        self.sample_ea.write_to_rp(self.ea_test1_rpath)
        self.ea1.write_to_rp(rp1_1)
        self.ea2.write_to_rp(rp1_2)
        self.ea4.write_to_rp(rp1_4)

        self.ea_test2_rpath.mkdir()
        rp2_1 = self.ea_test2_rpath.append('e1')
        rp2_2 = self.ea_test2_rpath.append('e2')
        rp2_3 = self.ea_test2_rpath.append('e3')
        list(map(rpath.RPath.touch, [rp2_1, rp2_2, rp2_3]))
        self.ea3.write_to_rp(self.ea_test2_rpath)
        self.sample_ea.write_to_rp(rp2_1)
        self.ea1.write_to_rp(rp2_2)
        self.ea2.write_to_rp(rp2_3)

        # just create an empty dir for tests
        if self.ea_empty_rpath.lstat():
            self.ea_empty_rpath.delete()
        self.ea_empty_rpath.mkdir()

    def testIterate(self):
        """Test writing several records and then reading them back"""
        self.make_backup_dirs()
        rp1 = self.ea_test1_rpath.append('e1')
        rp2 = self.ea_test1_rpath.append('e2')
        rp3 = self.ea_test1_rpath.append('e3')

        # Now write records corresponding to above rps into file
        Globals.rbdir = tempdir
        man = metadata.PatchDiffMan()
        writer = man.get_ea_writer('snapshot', 10000)
        for rp in [self.ea_test1_rpath, rp1, rp2, rp3]:
            ea = ExtendedAttributes(rp.index)
            ea.read_from_rp(rp)
            writer.write_object(ea)
        writer.close()

        # Read back records and compare
        ea_iter = man.get_eas_at_time(10000, None)
        assert ea_iter, "No extended_attributes.<time> file found"
        sample_ea_reread = next(ea_iter)
        # we ignore SELinux extended attributes for comparaison
        if sample_ea_reread.attr_dict:
            sample_ea_reread.attr_dict.pop(b'security.selinux', None)
        assert sample_ea_reread == self.sample_ea, "Re-read EAs %s are different from %s" % \
            (sample_ea_reread.attr_dict, self.sample_ea.attr_dict)
        ea1_reread = next(ea_iter)
        if ea1_reread.attr_dict:
            ea1_reread.attr_dict.pop(b'security.selinux', None)
        assert ea1_reread == self.ea1, "Re-read EAs %s are different from %s" % \
            (ea1_reread.attr_dict, self.ea1.attr_dict)
        ea2_reread = next(ea_iter)
        if ea2_reread.attr_dict:
            ea2_reread.attr_dict.pop(b'security.selinux', None)
        assert ea2_reread == self.ea2, "Re-read EAs %s are different from %s" % \
            (ea2_reread.attr_dict, self.ea2.attr_dict)
        ea3_reread = next(ea_iter)
        if ea3_reread.attr_dict:
            ea3_reread.attr_dict.pop(b'security.selinux', None)
        assert ea3_reread == self.ea3, "Re-read EAs %s are different from %s" % \
            (ea3_reread.attr_dict, self.ea3.attr_dict)
        try:
            next(ea_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Expected end to iterator"

    def testSeriesLocal(self):
        """Test backing up and restoring directories with EAs locally"""
        self.make_backup_dirs()
        dirlist = [
            self.ea_test1_dir, self.ea_empty_dir, self.ea_test2_dir,
            self.ea_test1_dir
        ]
        BackupRestoreSeries(1, 1, dirlist, compare_eas=1)

    def testSeriesRemote(self):
        """Test backing up, restoring directories with EA remotely"""
        self.make_backup_dirs()
        dirlist = [
            self.ea_test1_dir, self.ea_test2_dir, self.ea_empty_dir,
            self.ea_test1_dir
        ]
        BackupRestoreSeries(None, None, dirlist, compare_eas=1)

    def test_final_local(self):
        """Test backing up and restoring using 'rdiff-backup' script"""
        self.make_backup_dirs()
        self.make_temp_out_dirs()
        rdiff_backup(1,
                     1,
                     self.ea_test1_rpath.path,
                     tempdir.path,
                     current_time=10000)
        assert CompareRecursive(self.ea_test1_rpath, tempdir, compare_eas=1)

        rdiff_backup(1,
                     1,
                     self.ea_test2_rpath.path,
                     tempdir.path,
                     current_time=20000)
        assert CompareRecursive(self.ea_test2_rpath, tempdir, compare_eas=1)

        rdiff_backup(1,
                     1,
                     tempdir.path,
                     restore_dir.path,
                     extra_options=b'-r 10000')
        assert CompareRecursive(self.ea_test1_rpath,
                                restore_dir,
                                compare_eas=1)


class ACLTest(unittest.TestCase):
    """Test access control lists"""

    current_user = os.getenv('RDIFF_TEST_USER', pwd.getpwuid(os.getuid()).pw_name)
    current_group = os.getenv('RDIFF_TEST_GROUP', grp.getgrgid(os.getgid()).gr_name)

    sample_acl = AccessControlLists((), """user::rwx
user:root:rwx
group::r-x
group:root:r-x
mask::r-x
other::---""")
    dir_acl = AccessControlLists((), """user::rwx
user:root:rwx
group::r-x
group:root:r-x
mask::r-x
other::---
default:user::rwx
default:user:root:---
default:group::r-x
default:mask::r-x
default:other::---""")
    acl1 = AccessControlLists((b'a1', ), """user::r--
user:{0}:---
group::---
group:root:---
mask::---
other::---""".format(current_user))
    acl2 = AccessControlLists((b'a2', ), """user::rwx
group::r-x
group:{0}:rwx
mask::---
other::---""".format(current_group))
    acl3 = AccessControlLists((b'a3', ), """user::rwx
user:root:---
group::r-x
mask::---
other::---""")
    empty_acl = AccessControlLists((), "user::rwx\ngroup::---\nother::---")
    acl_test1_dir = os.path.join(abs_test_dir, b'acl_test1')
    acl_test1_rpath = rpath.RPath(Globals.local_connection, acl_test1_dir)
    acl_test2_dir = os.path.join(abs_test_dir, b'acl_test2')
    acl_test2_rpath = rpath.RPath(Globals.local_connection, acl_test2_dir)
    acl_empty_dir = os.path.join(abs_test_dir, b'acl_empty')
    acl_empty_rpath = rpath.RPath(Globals.local_connection, acl_empty_dir)

    def make_temp_out_dirs(self):
        """Make temp output and restore directories empty"""
        tempdir.setdata()  # in case the file changed in-between
        if tempdir.lstat():
            tempdir.delete()
        tempdir.mkdir()
        restore_dir.setdata()  # in case the file changed in-between
        if restore_dir.lstat():
            restore_dir.delete()

    def testBasic(self):
        """Test basic writing and reading of ACLs"""
        self.make_temp_out_dirs()
        new_acl = AccessControlLists(())
        tempdir.chmod(0o700)
        new_acl.read_from_rp(tempdir)
        assert new_acl.is_basic(), str(new_acl)
        assert not new_acl == self.sample_acl
        assert new_acl != self.sample_acl
        assert new_acl == self.empty_acl, \
            (str(new_acl), str(self.empty_acl))

        self.sample_acl.write_to_rp(tempdir)
        new_acl.read_from_rp(tempdir)
        assert str(new_acl) == str(self.sample_acl), \
            (str(new_acl), str(self.sample_acl))
        assert new_acl == self.sample_acl

    def testBasicDir(self):
        """Test reading and writing of ACL w/ defaults to directory"""
        self.make_temp_out_dirs()
        new_acl = AccessControlLists(())
        new_acl.read_from_rp(tempdir)
        assert new_acl.is_basic()
        assert new_acl != self.dir_acl

        self.dir_acl.write_to_rp(tempdir)
        new_acl.read_from_rp(tempdir)
        assert not new_acl.is_basic()
        if not new_acl == self.dir_acl:
            assert new_acl.eq_verbose(self.dir_acl)
            assert 0, "Shouldn't be here---eq != eq_verbose?"

    def testRecord(self):
        """Test writing a record and reading it back"""
        record = ACL2Record(self.sample_acl)
        new_acl = Record2ACL(record)
        if new_acl != self.sample_acl:
            print("New_acl", new_acl.entry_list)
            print("sample_acl", self.sample_acl.entry_list)
            print("New_acl text", str(new_acl))
            print("sample acl text", str(self.sample_acl))
            assert 0

        record2 = ACL2Record(self.dir_acl)
        new_acl2 = Record2ACL(record2)
        if not new_acl2 == self.dir_acl:
            assert new_acl2.eq_verbose(self.dir_acl)
            assert 0

    def testExtractor(self):
        """Test seeking inside a record list"""
        record_list = """# file: 0foo
user::r--
user:{0}:---
group::---
group:root:---
mask::---
other::---
# file: 1foo/bar/baz
user::r--
user:{0}:---
group::---
group:root:---
mask::---
other::---
# file: 2foo/\\012
user::r--
user:{0}:---
group::---
group:root:---
mask::---
other::---
""".format(self.current_user)
        extractor = ACLExtractor(io.BytesIO(os.fsencode(record_list)))
        acl_iter = extractor.iterate_starting_with(())
        first = next(acl_iter)
        assert first.index == (b'0foo', ), first
        second = next(acl_iter)
        assert second.index == (b'1foo', b'bar', b'baz'), second
        third = next(acl_iter)  # Test quoted filenames
        assert third.index == (b'2foo', b'\n'), third.index
        try:
            next(acl_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Too many elements in iterator"

        extractor = ACLExtractor(io.BytesIO(os.fsencode(record_list)))
        acl_iter = extractor.iterate_starting_with((b'1foo', b'bar'))
        assert next(acl_iter).index == (b'1foo', b'bar', b'baz')
        try:
            next(acl_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Too many elements in iterator"

    def make_backup_dirs(self):
        """Create testfiles/acl_test[12] directories"""
        if self.acl_test1_rpath.lstat():
            self.acl_test1_rpath.delete()
        self.acl_test1_rpath.mkdir()
        rp1_1 = self.acl_test1_rpath.append('a1')
        rp1_2 = self.acl_test1_rpath.append('a2')
        rp1_3 = self.acl_test1_rpath.append('a3')
        list(map(rpath.RPath.touch, [rp1_1, rp1_2, rp1_3]))
        self.dir_acl.write_to_rp(self.acl_test1_rpath)
        self.acl1.write_to_rp(rp1_1)
        self.acl2.write_to_rp(rp1_2)
        self.acl3.write_to_rp(rp1_3)

        if self.acl_test2_rpath.lstat():
            self.acl_test2_rpath.delete()
        self.acl_test2_rpath.mkdir()
        rp2_1, rp2_2, rp2_3 = list(
            map(self.acl_test2_rpath.append, ('a1', 'a2', 'a3')))
        list(map(rpath.RPath.touch, (rp2_1, rp2_2, rp2_3)))
        self.sample_acl.write_to_rp(self.acl_test2_rpath)
        self.acl3.write_to_rp(rp2_1)
        self.acl1.write_to_rp(rp2_2)
        self.acl2.write_to_rp(rp2_3)

        # just create an empty dir for tests
        if self.acl_empty_rpath.lstat():
            self.acl_empty_rpath.delete()
        self.acl_empty_rpath.mkdir()

    def testIterate(self):
        """Test writing several records and then reading them back"""
        self.make_backup_dirs()
        self.make_temp_out_dirs()
        rp1 = self.acl_test1_rpath.append('a1')
        rp2 = self.acl_test1_rpath.append('a2')
        rp3 = self.acl_test1_rpath.append('a3')

        # Now write records corresponding to above rps into file
        Globals.rbdir = tempdir
        man = metadata.PatchDiffMan()
        writer = man.get_acl_writer('snapshot', 10000)
        for rp in [self.acl_test1_rpath, rp1, rp2, rp3]:
            acl = AccessControlLists(rp.index)
            acl.read_from_rp(rp)
            writer.write_object(acl)
        writer.close()

        # Read back records and compare
        acl_iter = man.get_acls_at_time(10000, None)
        assert acl_iter, "No acl file found"
        dir_acl_reread = next(acl_iter)
        assert dir_acl_reread == self.dir_acl
        acl1_reread = next(acl_iter)
        assert acl1_reread == self.acl1
        acl2_reread = next(acl_iter)
        assert acl2_reread == self.acl2
        acl3_reread = next(acl_iter)
        assert acl3_reread == self.acl3
        try:
            extra = next(acl_iter)
        except StopIteration:
            pass
        else:
            assert 0, "Got unexpected object: " + repr(extra)

    def testSeriesLocal(self):
        """Test backing up and restoring directories with ACLs locally"""
        self.make_backup_dirs()
        dirlist = [
            self.acl_test1_dir, self.acl_empty_dir, self.acl_test2_dir,
            self.acl_test1_dir
        ]
        BackupRestoreSeries(1, 1, dirlist, compare_acls=1)

    def testSeriesRemote(self):
        """Test backing up, restoring directories with EA remotely"""
        self.make_backup_dirs()
        dirlist = [
            self.acl_test1_dir, self.acl_test2_dir, self.acl_empty_dir,
            self.acl_test1_dir
        ]
        BackupRestoreSeries(None, None, dirlist, compare_acls=1)

    def test_final_local(self):
        """Test backing up and restoring using 'rdiff-backup' script"""
        self.make_backup_dirs()
        self.make_temp_out_dirs()
        rdiff_backup(1,
                     1,
                     self.acl_test1_rpath.path,
                     tempdir.path,
                     current_time=10000)
        assert CompareRecursive(self.acl_test1_rpath, tempdir, compare_acls=1)

        rdiff_backup(1,
                     1,
                     self.acl_test2_rpath.path,
                     tempdir.path,
                     current_time=20000)
        assert CompareRecursive(self.acl_test2_rpath, tempdir, compare_acls=1)

        rdiff_backup(1,
                     1,
                     tempdir.path,
                     restore_dir.path,
                     extra_options=b'-r 10000')
        assert CompareRecursive(self.acl_test1_rpath,
                                restore_dir,
                                compare_acls=1)

        restore_dir.delete()
        rdiff_backup(1,
                     1,
                     tempdir.path,
                     restore_dir.path,
                     extra_options=b'-r now')
        assert CompareRecursive(self.acl_test2_rpath,
                                restore_dir,
                                compare_acls=1)

    def test_acl_mapping(self):
        """Test mapping ACL names"""

        def make_dir(rootrp):
            if rootrp.lstat():
                rootrp.delete()
            rootrp.mkdir()
            rp = rootrp.append('a1')
            rp.touch()
            acl = AccessControlLists(('a1', ), """user::rwx
user:root:rwx
user:{0}:---
user:bin:r--
group::r-x
group:root:r-x
group:{1}:-w-
mask::r-x
other::---""".format(self.current_user, self.current_group))
            rp.write_acl(acl)
            return rp

        def write_mapping_file(rootrp):
            map_rp = rootrp.append('mapping_file')
            map_rp.write_string("root:{1}\n{0}:bin\nbin:root".format(
                self.current_user, self.current_group))
            return map_rp

        def get_perms_of_user(acl, user):
            """Return the permissions of ACL_USER in acl, or None"""
            for typechar, owner_pair, perms in acl.entry_list:
                if typechar == "u" and owner_pair[1] == user:
                    return perms
            return None

        self.make_temp_out_dirs()
        rootrp = rpath.RPath(Globals.local_connection,
                             os.path.join(abs_test_dir, b'acl_map_test'))
        make_dir(rootrp)
        map_rp = write_mapping_file(rootrp)

        rdiff_backup(1,
                     1,
                     rootrp.path,
                     tempdir.path,
                     extra_options=b"--user-mapping-file %b" % (map_rp.path, ))

        out_rp = tempdir.append('a1')
        assert out_rp.isreg()
        out_acl = tempdir.append('a1').get_acl()
        assert get_perms_of_user(out_acl, 'root') == 4
        assert get_perms_of_user(out_acl, self.current_user) == 7
        assert get_perms_of_user(out_acl, 'bin') == 0

    def test_acl_dropping(self):
        """Test dropping of ACL names"""
        self.make_temp_out_dirs()
        rp = tempdir.append('a1')
        rp.touch()
        """ben uses a dvorak keyboard, and these sequences are
        analogous to asdfsjkd for a qwerty user... these
        users and groups are not expected to exist. -dean"""
        acl = AccessControlLists(('a1', ), """user::rwx
user:aoensutheu:r--
group::r-x
group:aeuai:r-x
group:enutohnh:-w-
other::---""")
        rp.write_acl(acl)
        rp2 = tempdir.append('a1')
        acl2 = AccessControlLists(('a1', ))
        acl2.read_from_rp(rp2)
        assert acl2.is_basic()
        Globals.never_drop_acls = 1
        try:
            rp.write_acl(acl)
        except SystemExit:
            pass
        else:
            assert 0, "Above should have exited with fatal error"
        Globals.never_drop_acls = None

    def test_nochange(self):
        """Make sure files with ACLs not unnecessarily flagged changed"""
        self.make_temp_out_dirs()
        self.make_backup_dirs()
        rdiff_backup(1,
                     1,
                     self.acl_test1_rpath.path,
                     tempdir.path,
                     current_time=10000)
        rdiff_backup(1,
                     1,
                     self.acl_test1_rpath.path,
                     tempdir.path,
                     current_time=20000)
        incdir = tempdir.append('rdiff-backup-data', 'increments')
        assert incdir.isdir(), incdir
        assert not incdir.listdir(), incdir.listdir()


class CombinedTest(unittest.TestCase):
    """Test backing up and restoring directories with both EAs and ACLs"""
    combo_test1_dir = os.path.join(abs_test_dir, b'ea_acl_test1')
    combo_test1_rpath = rpath.RPath(Globals.local_connection, combo_test1_dir)
    combo_test2_dir = os.path.join(abs_test_dir, b'ea_acl_test2')
    combo_test2_rpath = rpath.RPath(Globals.local_connection, combo_test2_dir)
    combo_empty_dir = os.path.join(abs_test_dir, b'ea_acl_empty')
    combo_empty_rpath = rpath.RPath(Globals.local_connection, combo_empty_dir)

    def make_backup_dirs(self):
        """Create testfiles/ea_acl_test[12] directories"""
        if self.combo_test1_rpath.lstat():
            self.combo_test1_rpath.delete()
        if self.combo_test2_rpath.lstat():
            self.combo_test2_rpath.delete()
        self.combo_test1_rpath.mkdir()
        rp1_1, rp1_2, rp1_3 = list(
            map(self.combo_test1_rpath.append, ('c1', 'c2', 'c3')))
        list(map(rpath.RPath.touch, [rp1_1, rp1_2, rp1_3]))
        ACLTest.dir_acl.write_to_rp(self.combo_test1_rpath)
        EATest.sample_ea.write_to_rp(self.combo_test1_rpath)
        ACLTest.acl1.write_to_rp(rp1_1)
        EATest.ea2.write_to_rp(rp1_2)
        ACLTest.acl3.write_to_rp(rp1_3)
        EATest.ea3.write_to_rp(rp1_3)

        self.combo_test2_rpath.mkdir()
        rp2_1, rp2_2, rp2_3 = list(
            map(self.combo_test2_rpath.append, ('c1', 'c2', 'c3')))
        list(map(rpath.RPath.touch, [rp2_1, rp2_2, rp2_3]))
        ACLTest.sample_acl.write_to_rp(self.combo_test2_rpath)
        EATest.ea1.write_to_rp(rp2_1)
        EATest.ea3.write_to_rp(rp2_2)
        ACLTest.acl2.write_to_rp(rp2_2)

        # just create an empty dir for tests
        if self.combo_empty_rpath.lstat():
            self.combo_empty_rpath.delete()
        self.combo_empty_rpath.mkdir()

    def testSeriesLocal(self):
        """Test backing up and restoring EAs/ACLs locally"""
        self.make_backup_dirs()
        dirlist = [
            self.combo_test1_dir, self.combo_test2_dir, self.combo_empty_dir,
            self.combo_test1_dir
        ]
        BackupRestoreSeries(1, 1, dirlist, compare_eas=1, compare_acls=1)

    def testSeriesRemote(self):
        """Test backing up and restoring EAs/ACLs locally"""
        self.make_backup_dirs()
        dirlist = [
            self.combo_test1_dir, self.combo_empty_dir, self.combo_test2_dir,
            self.combo_test1_dir
        ]
        BackupRestoreSeries(None, None, dirlist, compare_eas=1, compare_acls=1)


if __name__ == "__main__":
    unittest.main()



"""commontest - Some functions and constants common to several test cases.
Can be called also directly to setup the test environment"""
import os
import sys
import code
import shutil
import subprocess
# Avoid circularities
from rdiff_backup.log import Log
from rdiff_backup import Globals, Hardlink, SetConnections, Main, \
    selection, rpath, eas_acls, rorpiter, Security, hash

RBBin = os.fsencode(shutil.which("rdiff-backup"))

# Working directory is defined by Tox, venv or the current build directory
abs_work_dir = os.getenvb(
    b'TOX_ENV_DIR',
    os.getenvb(b'VIRTUAL_ENV', os.path.join(os.getcwdb(), b'build')))
abs_test_dir = os.path.join(abs_work_dir, b'testfiles')
abs_output_dir = os.path.join(abs_test_dir, b'output')
abs_restore_dir = os.path.join(abs_test_dir, b'restore')

# the directory with the testfiles used as input is in the parent directory of the Git clone
old_test_dir = os.path.join(os.path.dirname(os.getcwdb()),
                            b'rdiff-backup_testfiles')
old_inc1_dir = os.path.join(old_test_dir, b'increment1')
old_inc2_dir = os.path.join(old_test_dir, b'increment2')
old_inc3_dir = os.path.join(old_test_dir, b'increment3')
old_inc4_dir = os.path.join(old_test_dir, b'increment4')

# the directory in which all testing scripts are placed is the one
abs_testing_dir = os.path.dirname(os.path.abspath(os.fsencode(sys.argv[0])))

__no_execute__ = 1  # Keeps the actual rdiff-backup program from running


def Myrm(dirstring):
    """Run myrm on given directory string"""
    root_rp = rpath.RPath(Globals.local_connection, dirstring)
    for rp in selection.Select(root_rp).set_iter():
        if rp.isdir():
            rp.chmod(0o700)  # otherwise may not be able to remove
    assert not os.system(b"rm -rf %s" % (root_rp.path, ))


def re_init_rpath_dir(rp, uid=-1, gid=-1):
    """Delete directory if present, then recreate"""
    if rp.lstat():
        Myrm(rp.path)
        rp.setdata()
    rp.mkdir()
    rp.chown(uid, gid)


def re_init_subdir(maindir, subdir):
    """Remove a sub-directory and return its name joined
    to the main directory as an empty directory"""
    dir = os.path.join(maindir, subdir)
    Myrm(dir)
    os.makedirs(dir)
    return dir


# two temporary directories to simulate remote actions
abs_remote1_dir = re_init_subdir(abs_test_dir, b'remote1')
abs_remote2_dir = re_init_subdir(abs_test_dir, b'remote2')


def MakeOutputDir():
    """Initialize the output directory"""
    Myrm(abs_output_dir)
    rp = rpath.RPath(Globals.local_connection, abs_output_dir)
    rp.mkdir()
    return rp


def rdiff_backup(source_local,
                 dest_local,
                 src_dir,
                 dest_dir,
                 current_time=None,
                 extra_options=b"",
                 input=None,
                 check_return_val=1,
                 expected_ret_val=0):
    """Run rdiff-backup with the given options

    source_local and dest_local are boolean values.  If either is
    false, then rdiff-backup will be run pretending that src_dir and
    dest_dir, respectively, are remote.  The server process will be
    run in directories remote1 and remote2 respectively.

    src_dir and dest_dir are the source and destination
    (mirror) directories, relative to the testing directory.

    If current time is true, add the --current-time option with the
    given number of seconds.

    extra_options are just added to the command line.

    """
    if not source_local:
        src_dir = (b"'cd %s; %s --server'::%s" %
                   (abs_remote1_dir, RBBin, src_dir))
    if dest_dir and not dest_local:
        dest_dir = (b"'cd %s; %s --server'::%s" %
                    (abs_remote2_dir, RBBin, dest_dir))

    cmdargs = [RBBin, extra_options]
    if not (source_local and dest_local):
        cmdargs.append(b"--remote-schema %s")

    if current_time:
        cmdargs.append(b"--current-time %i" % current_time)
    cmdargs.append(src_dir)
    if dest_dir:
        cmdargs.append(dest_dir)
    cmdline = b" ".join(cmdargs)
    print("Executing: ", cmdline)
    ret_val = subprocess.run(cmdline,
                             shell=True,
                             input=input,
                             universal_newlines=False).returncode
    if check_return_val:
        # the construct is needed because os.system seemingly doesn't
        # respect expected return values (FIXME)
        assert ((expected_ret_val == 0 and ret_val == 0) or (expected_ret_val > 0 and ret_val > 0)), \
            "Return code %d of command `%a` isn't expected %d." % \
            (ret_val, cmdline, expected_ret_val)
    return ret_val


def InternalBackup(source_local,
                   dest_local,
                   src_dir,
                   dest_dir,
                   current_time=None,
                   eas=None,
                   acls=None):
    """Backup src to dest internally

    This is like rdiff_backup but instead of running a separate
    rdiff-backup script, use the separate *.py files.  This way the
    script doesn't have to be rebuild constantly, and stacktraces have
    correct line/file references.

    """
    Globals.current_time = current_time
    Globals.security_level = "override"
    remote_schema = b'%s'

    if not source_local:
        src_dir = b"cd %s; %s/server.py::%s" % (abs_remote1_dir, abs_testing_dir, src_dir)
    if not dest_local:
        dest_dir = b"cd %s; %s/server.py::%s" % (abs_remote2_dir, abs_testing_dir, dest_dir)

    cmdpairs = SetConnections.get_cmd_pairs([src_dir, dest_dir], remote_schema)
    Security.initialize("backup", cmdpairs)
    rpin, rpout = list(map(SetConnections.cmdpair2rp, cmdpairs))
    for attr in ('eas_active', 'eas_write', 'eas_conn'):
        SetConnections.UpdateGlobal(attr, eas)
    for attr in ('acls_active', 'acls_write', 'acls_conn'):
        SetConnections.UpdateGlobal(attr, acls)
    Main.misc_setup([rpin, rpout])
    Main.Backup(rpin, rpout)
    Main.cleanup()


def InternalMirror(source_local, dest_local, src_dir, dest_dir):
    """Mirror src to dest internally

    like InternalBackup, but only mirror.  Do this through
    InternalBackup, but then delete rdiff-backup-data directory.

    """
    # Save attributes of root to restore later
    src_root = rpath.RPath(Globals.local_connection, src_dir)
    dest_root = rpath.RPath(Globals.local_connection, dest_dir)
    dest_rbdir = dest_root.append("rdiff-backup-data")

    InternalBackup(source_local, dest_local, src_dir, dest_dir)
    dest_root.setdata()
    Myrm(dest_rbdir.path)
    # Restore old attributes
    rpath.copy_attribs(src_root, dest_root)


def InternalRestore(mirror_local,
                    dest_local,
                    mirror_dir,
                    dest_dir,
                    time,
                    eas=None,
                    acls=None):
    """Restore mirror_dir to dest_dir at given time

    This will automatically find the increments.XXX.dir representing
    the time specified.  The mirror_dir and dest_dir are relative to
    the testing directory and will be modified for remote trials.

    """
    Main.force = 1
    Main.restore_root_set = 0
    remote_schema = b'%s'
    Globals.security_level = "override"
    if not mirror_local:
        mirror_dir = b"cd %s; %s/server.py::%s" % (abs_remote1_dir, abs_testing_dir, mirror_dir)
    if not dest_local:
        dest_dir = b"cd %s; %s/server.py::%s" % (abs_remote2_dir, abs_testing_dir, dest_dir)

    cmdpairs = SetConnections.get_cmd_pairs([mirror_dir, dest_dir],
                                            remote_schema)
    Security.initialize("restore", cmdpairs)
    mirror_rp, dest_rp = list(map(SetConnections.cmdpair2rp, cmdpairs))
    for attr in ('eas_active', 'eas_write', 'eas_conn'):
        SetConnections.UpdateGlobal(attr, eas)
    for attr in ('acls_active', 'acls_write', 'acls_conn'):
        SetConnections.UpdateGlobal(attr, acls)
    Main.misc_setup([mirror_rp, dest_rp])
    inc = get_increment_rp(mirror_rp, time)
    if inc:
        Main.Restore(get_increment_rp(mirror_rp, time), dest_rp)
    else:  # use alternate syntax
        Main.restore_timestr = str(time)
        Main.Restore(mirror_rp, dest_rp, restore_as_of=1)
    Main.cleanup()


def get_increment_rp(mirror_rp, time):
    """Return increment rp matching time in seconds"""
    data_rp = mirror_rp.append("rdiff-backup-data")
    if not data_rp.isdir():
        return None
    for filename in data_rp.listdir():
        rp = data_rp.append(filename)
        if rp.isincfile() and rp.getincbase_bname() == b"increments":
            if rp.getinctime() == time:
                return rp
    return None  # Couldn't find appropriate increment


def _reset_connections(src_rp, dest_rp):
    """Reset some global connection information"""
    Globals.security_level = "override"
    Globals.isbackup_reader = Globals.isbackup_writer = None
    SetConnections.UpdateGlobal('rbdir', None)
    Main.misc_setup([src_rp, dest_rp])


def CompareRecursive(src_rp,
                     dest_rp,
                     compare_hardlinks=1,
                     equality_func=None,
                     exclude_rbdir=1,
                     ignore_tmp_files=None,
                     compare_ownership=0,
                     compare_eas=0,
                     compare_acls=0):
    """Compare src_rp and dest_rp, which can be directories

    This only compares file attributes, not the actual data.  This
    will overwrite the hardlink dictionaries if compare_hardlinks is
    specified.

    """

    def get_selection_functions():
        """Return generators of files in source, dest"""
        src_rp.setdata()
        dest_rp.setdata()
        src_select = selection.Select(src_rp)
        dest_select = selection.Select(dest_rp)

        if ignore_tmp_files:
            # Ignoring temp files can be useful when we want to check the
            # correctness of a backup which aborted in the middle.  In
            # these cases it is OK to have tmp files lying around.
            src_select.add_selection_func(
                src_select.regexp_get_sf(".*rdiff-backup.tmp.[^/]+$", 0))
            dest_select.add_selection_func(
                dest_select.regexp_get_sf(".*rdiff-backup.tmp.[^/]+$", 0))

        if exclude_rbdir:  # Exclude rdiff-backup-data directory
            src_select.parse_rbdir_exclude()
            dest_select.parse_rbdir_exclude()

        return src_select.set_iter(), dest_select.set_iter()

    def hardlink_rorp_eq(src_rorp, dest_rorp):
        Hardlink.add_rorp(dest_rorp)
        Hardlink.add_rorp(src_rorp, dest_rorp)
        rorp_eq = Hardlink.rorp_eq(src_rorp, dest_rorp)
        if not src_rorp.isreg() or not dest_rorp.isreg() or src_rorp.getnumlinks() == dest_rorp.getnumlinks() == 1:
            if not rorp_eq:
                Log("Hardlink compare error with when no links exist exist", 3)
                Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
                Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
                return 0
        elif src_rorp.getnumlinks() > 1 and not Hardlink.islinked(src_rorp):
            if rorp_eq:
                Log("Hardlink compare error with first linked src_rorp and no dest_rorp sha1", 3)
                Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
                Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
                return 0
            hash.compute_sha1(dest_rorp)
            rorp_eq = Hardlink.rorp_eq(src_rorp, dest_rorp)
            if src_rorp.getnumlinks() != dest_rorp.getnumlinks():
                if rorp_eq:
                    Log("Hardlink compare error with first linked src_rorp, with dest_rorp sha1, and with differing link counts", 3)
                    Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
                    Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
                    return 0
            elif not rorp_eq:
                Log("Hardlink compare error with first linked src_rorp, with dest_rorp sha1, and with equal link counts", 3)
                Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
                Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
                return 0
        elif src_rorp.getnumlinks() != dest_rorp.getnumlinks():
            if rorp_eq:
                Log("Hardlink compare error with non-first linked src_rorp and with differing link counts", 3)
                Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
                Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
                return 0
        elif not rorp_eq:
            Log("Hardlink compare error with non-first linked src_rorp and with equal link counts", 3)
            Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
            Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
            return 0
        Hardlink.del_rorp(src_rorp)
        Hardlink.del_rorp(dest_rorp)
        return 1

    def equality_func(src_rorp, dest_rorp):
        """Combined eq func returns true if two files compare same"""
        if not src_rorp:
            Log("Source rorp missing: %s" % str(dest_rorp), 3)
            return 0
        if not dest_rorp:
            Log("Dest rorp missing: %s" % str(src_rorp), 3)
            return 0
        if not src_rorp.equal_verbose(dest_rorp,
                                      compare_ownership=compare_ownership):
            return 0
        if compare_hardlinks and not hardlink_rorp_eq(src_rorp, dest_rorp):
            return 0
        if compare_eas and not eas_acls.ea_compare_rps(src_rorp, dest_rorp):
            Log(
                "Different EAs in files %s and %s" %
                (src_rorp.get_indexpath(), dest_rorp.get_indexpath()), 3)
            return 0
        if compare_acls and not eas_acls.acl_compare_rps(src_rorp, dest_rorp):
            Log(
                "Different ACLs in files %s and %s" %
                (src_rorp.get_indexpath(), dest_rorp.get_indexpath()), 3)
            return 0
        return 1

    Log(
        "Comparing %s and %s, hardlinks %s, eas %s, acls %s" %
        (src_rp.get_safepath(), dest_rp.get_safepath(), compare_hardlinks,
         compare_eas, compare_acls), 3)
    if compare_hardlinks:
        reset_hardlink_dicts()
    src_iter, dest_iter = get_selection_functions()
    for src_rorp, dest_rorp in rorpiter.Collate2Iters(src_iter, dest_iter):
        if not equality_func(src_rorp, dest_rorp):
            return 0
    return 1

    def rbdir_equal(src_rorp, dest_rorp):
        """Like hardlink_equal, but make allowances for data directories"""
        if not src_rorp.index and not dest_rorp.index:
            return 1
        if (src_rorp.index and src_rorp.index[0] == 'rdiff-backup-data' and src_rorp.index == dest_rorp.index):
            # Don't compare dirs - they don't carry significant info
            if dest_rorp.isdir() and src_rorp.isdir():
                return 1
            if dest_rorp.isreg() and src_rorp.isreg():
                # Don't compare gzipped files because it is apparently
                # non-deterministic.
                if dest_rorp.index[-1].endswith('gz'):
                    return 1
                # Don't compare .missing increments because they don't matter
                if dest_rorp.index[-1].endswith('.missing'):
                    return 1
        if compare_eas and not eas_acls.ea_compare_rps(src_rorp, dest_rorp):
            Log("Different EAs in files %s and %s" %
                (src_rorp.get_indexpath(), dest_rorp.get_indexpath()))
            return None
        if compare_acls and not eas_acls.acl_compare_rps(src_rorp, dest_rorp):
            Log(
                "Different ACLs in files %s and %s" %
                (src_rorp.get_indexpath(), dest_rorp.get_indexpath()), 3)
            return None
        if compare_hardlinks:
            if Hardlink.rorp_eq(src_rorp, dest_rorp):
                return 1
        elif src_rorp.equal_verbose(dest_rorp,
                                    compare_ownership=compare_ownership):
            return 1
        Log("%s: %s" % (src_rorp.index, Hardlink.get_inode_key(src_rorp)), 3)
        Log("%s: %s" % (dest_rorp.index, Hardlink.get_inode_key(dest_rorp)), 3)
        return None


def reset_hardlink_dicts():
    """Clear the hardlink dictionaries"""
    Hardlink._inode_index = {}


def BackupRestoreSeries(source_local,
                        dest_local,
                        list_of_dirnames,
                        compare_hardlinks=1,
                        dest_dirname=abs_output_dir,
                        restore_dirname=abs_restore_dir,
                        compare_backups=1,
                        compare_eas=0,
                        compare_acls=0,
                        compare_ownership=0):
    """Test backing up/restoring of a series of directories

    The dirnames correspond to a single directory at different times.
    After each backup, the dest dir will be compared.  After the whole
    set, each of the earlier directories will be recovered to the
    restore_dirname and compared.

    """
    Globals.set('preserve_hardlinks', compare_hardlinks)
    time = 10000
    dest_rp = rpath.RPath(Globals.local_connection, dest_dirname)
    restore_rp = rpath.RPath(Globals.local_connection, restore_dirname)

    Myrm(dest_dirname)
    for dirname in list_of_dirnames:
        src_rp = rpath.RPath(Globals.local_connection, dirname)
        reset_hardlink_dicts()
        _reset_connections(src_rp, dest_rp)

        InternalBackup(source_local,
                       dest_local,
                       dirname,
                       dest_dirname,
                       time,
                       eas=compare_eas,
                       acls=compare_acls)
        time += 10000
        _reset_connections(src_rp, dest_rp)
        if compare_backups:
            assert CompareRecursive(src_rp,
                                    dest_rp,
                                    compare_hardlinks,
                                    compare_eas=compare_eas,
                                    compare_acls=compare_acls,
                                    compare_ownership=compare_ownership)

    time = 10000
    for dirname in list_of_dirnames[:-1]:
        reset_hardlink_dicts()
        Myrm(restore_dirname)
        InternalRestore(dest_local,
                        source_local,
                        dest_dirname,
                        restore_dirname,
                        time,
                        eas=compare_eas,
                        acls=compare_acls)
        src_rp = rpath.RPath(Globals.local_connection, dirname)
        assert CompareRecursive(src_rp,
                                restore_rp,
                                compare_eas=compare_eas,
                                compare_acls=compare_acls,
                                compare_ownership=compare_ownership)

        # Restore should default back to newest time older than it
        # with a backup then.
        if time == 20000:
            time = 21000

        time += 10000


def MirrorTest(source_local,
               dest_local,
               list_of_dirnames,
               compare_hardlinks=1,
               dest_dirname=abs_output_dir):
    """Mirror each of list_of_dirnames, and compare after each"""
    Globals.set('preserve_hardlinks', compare_hardlinks)
    dest_rp = rpath.RPath(Globals.local_connection, dest_dirname)
    old_force_val = Main.force
    Main.force = 1

    Myrm(dest_dirname)
    for dirname in list_of_dirnames:
        src_rp = rpath.RPath(Globals.local_connection, dirname)
        reset_hardlink_dicts()
        _reset_connections(src_rp, dest_rp)

        InternalMirror(source_local, dest_local, dirname, dest_dirname)
        _reset_connections(src_rp, dest_rp)
        assert CompareRecursive(src_rp, dest_rp, compare_hardlinks)
    Main.force = old_force_val


def raise_interpreter(use_locals=None):
    """Start Python interpreter, with local variables if locals is true"""
    if use_locals:
        local_dict = locals()
    else:
        local_dict = globals()
    code.InteractiveConsole(local_dict).interact()


def getrefs(i, depth):
    """Get the i'th object in memory, return objects that reference it"""
    import sys
    import gc
    import types
    o = sys.getobjects(i)[-1]
    for d in range(depth):
        for ref in gc.get_referrers(o):
            if type(ref) in (list, dict, types.InstanceType):
                if type(ref) is dict and 'copyright' in ref:
                    continue
                o = ref
                break
        else:
            print("Max depth ", d)
            return o
    return o


def iter_equal(iter1, iter2, verbose=None, operator=lambda x, y: x == y):
    """True if iterator 1 has same elements as iterator 2

    Use equality operator, or == if it is unspecified.

    """
    for i1 in iter1:
        try:
            i2 = next(iter2)
        except StopIteration:
            if verbose:
                print("End when i1 = %s" % (i1, ))
            return False
        if not operator(i1, i2):
            if verbose:
                print("%s not equal to %s" % (i1, i2))
            return False
    try:
        i2 = next(iter2)
    except StopIteration:
        return True
    if verbose:
        print("End when i2 = %s" % (i2, ))
    return False


def iter_map(function, iterator):
    """Like map in a lazy functional programming language"""
    for i in iterator:
        yield function(i)


if __name__ == '__main__':
    os.makedirs(abs_test_dir, exist_ok=True)



import unittest
import time
from commontest import *  # noqa: F403, F401 some side effect or test fails
from rdiff_backup import Globals, Time


class TimeTest(unittest.TestCase):
    def testConversion(self):
        """test timetostring and stringtotime"""
        Time.setcurtime()
        assert type(Time.curtime) is float or int, Time.curtime
        assert type(Time.curtimestr) is str, Time.curtimestr
        assert (Time.cmp(int(Time.curtime),
                         Time.curtimestr) == 0 or Time.cmp(int(Time.curtime) + 1,
                                                           Time.curtimestr) == 0)
        time.sleep(1.05)
        assert Time.cmp(time.time(), Time.curtime) == 1
        assert Time.cmp(Time.timetostring(time.time()), Time.curtimestr) == 1

    def testConversion_separator(self):
        """Same as testConversion, but change time Separator"""
        Globals.time_separator = "_"
        self.testConversion()
        Globals.time_separator = ":"

    def testCmp(self):
        """Test time comparisons"""
        cmp = Time.cmp
        assert cmp(1, 2) == -1
        assert cmp(2, 2) == 0
        assert cmp(5, 1) == 1
        assert cmp("2001-09-01T21:49:04Z", "2001-08-01T21:49:04Z") == 1
        assert cmp("2001-09-01T04:49:04+03:23", "2001-09-01T21:49:04Z") == -1
        assert cmp("2001-09-01T12:00:00Z", "2001-09-01T04:00:00-08:00") == 0
        assert cmp("2001-09-01T12:00:00-08:00",
                   "2001-09-01T12:00:00-07:00") == 1

    def testBytestotime(self):
        """Test converting byte string to time"""
        timesec = int(time.time())
        assert timesec == int(Time.bytestotime(Time.timetostring(timesec).encode('ascii')))

        # assure that non-ascii byte strings return None and that they don't
        # throw an exception (issue #295)
        assert Time.bytestotime(b'\xff') is None

    def testStringtotime(self):
        """Test converting string to time"""
        timesec = int(time.time())
        assert timesec == int(Time.stringtotime(Time.timetostring(timesec)))
        assert not Time.stringtotime("2001-18-83T03:03:03Z")
        assert not Time.stringtotime("2001-01-23L03:03:03L")
        assert not Time.stringtotime("2001_01_23T03:03:03Z")

    def testIntervals(self):
        """Test converting strings to intervals"""
        i2s = Time.intstringtoseconds
        for s in ["32", "", "d", "231I", "MM", "s", "-2h"]:
            try:
                i2s(s)
            except Time.TimeException:
                pass
            else:
                assert 0, s
        assert i2s("7D") == 7 * 86400
        assert i2s("232s") == 232
        assert i2s("2M") == 2 * 30 * 86400
        assert i2s("400m") == 400 * 60
        assert i2s("1Y") == 365 * 86400
        assert i2s("30h") == 30 * 60 * 60
        assert i2s("3W") == 3 * 7 * 86400

    def testIntervalsComposite(self):
        """Like above, but allow composite intervals"""
        i2s = Time.intstringtoseconds
        assert i2s("7D2h") == 7 * 86400 + 2 * 3600
        assert i2s("2Y3s") == 2 * 365 * 86400 + 3
        assert i2s("1M2W4D2h5m20s") == \
            (30 * 86400 + 2 * 7 * 86400 + 4 * 86400 + 2 * 3600 + 5 * 60 + 20)

    def testPrettyIntervals(self):
        """Test printable interval conversion"""
        assert Time.inttopretty(3600) == "1 hour"
        assert Time.inttopretty(7220) == "2 hours 20 seconds"
        assert Time.inttopretty(0) == "0 seconds"
        assert Time.inttopretty(353) == "5 minutes 53 seconds"
        assert Time.inttopretty(3661) == "1 hour 1 minute 1 second"
        assert Time.inttopretty(353.234234) == "5 minutes 53.23 seconds"

    def testPrettyTimes(self):
        """Convert seconds to pretty and back"""
        now = int(time.time())
        for i in [1, 200000, now]:
            assert Time.prettytotime(Time.timetopretty(i)) == i, i
        assert Time.prettytotime("now") is None
        assert Time.prettytotime("12314") is None

    def testGenericString(self):
        """Test genstrtotime, conversion of arbitrary string to time"""
        g2t = Time.genstrtotime
        assert g2t('now', 1000) == 1000
        assert g2t('2h3s', 10000) == 10000 - 2 * 3600 - 3
        assert g2t('2001-09-01T21:49:04Z') == \
            Time.stringtotime('2001-09-01T21:49:04Z')
        assert g2t('2002-04-26T04:22:01') == \
            Time.stringtotime('2002-04-26T04:22:01' + Time.gettzd())
        t = Time.stringtotime('2001-05-12T00:00:00' + Time.gettzd())
        assert g2t('2001-05-12') == t
        assert g2t('2001/05/12') == t
        assert g2t('5/12/2001') == t
        assert g2t('123456') == 123456

    def testGenericStringErrors(self):
        """Test genstrtotime on some bad strings"""
        g2t = Time.genstrtotime
        self.assertRaises(Time.TimeException, g2t, "hello")
        self.assertRaises(Time.TimeException, g2t, "")
        self.assertRaises(Time.TimeException, g2t, "3q")

    def testTimeZone(self):
        """Test stringtotime on two strings straddling timezones"""
        f = Time.stringtotime
        invf = Time.timetostring
        s1 = "2005-04-03T03:45:03-03:00"
        s2 = "2005-04-03T02:45:03-03:00"
        diff = f(s1) - f(s2)
        assert diff == 3600, diff

        assert f(invf(f(s1))) == f(s1), (s1, invf(f(s1)), f(invf(f(s1))),
                                         f(s1))
        assert f(invf(f(s2))) == f(s2), (s2, f(invf(f(s2))), f(s2))


if __name__ == '__main__':
    unittest.main()



# Copyright 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA
"""Handle long filenames

rdiff-backup sometimes wants to write filenames longer than allowed by
the destination directory.  This can happen in 3 ways:

1)  Because the destination directory has a low maximum length limit.
2)  When the source directory has a filename close to the limit, so
    that its increments would be above the limit.
3)  When quoting is enabled, so that even the mirror filenames are too
    long.

When rdiff-backup would otherwise write a file whose name is too long,
instead it either skips the operation altogether (for non-regular
files), or writes the data to a unique file in the
rdiff-backup-data/long-filename directory.  This file will have an
arbitrary basename, but if it's an increment the suffix will be the
same.  The name will be recorded in the mirror_metadata so we can find
it later.

"""

import errno
from . import log, Globals, restore, regress

long_name_dir = b"long_filename_data"
rootrp = None


def get_long_rp(base=None):
    """Return an rpath in long name directory with given base"""
    global rootrp
    if not rootrp:
        rootrp = Globals.rbdir.append(long_name_dir)
        if not rootrp.lstat():
            rootrp.mkdir()
    if base:
        return rootrp.append(base)
    else:
        return rootrp


# ------------------------------------------------------------------
# These functions used mainly for backing up

# integer number of next free prefix.  Names will be created from
# integers consecutively like '1', '2', and so on.
free_name_counter = None

# Filename which holds the next available free name in it
counter_filename = b"next_free"


def get_next_free():
    """Return next free filename available in the long filename directory"""
    global free_name_counter

    def scan_next_free():
        """Return value of free_name_counter by listing long filename dir"""
        log.Log("Setting next free from long filenames dir", 5)
        cur_high = 0
        for filename in get_long_rp().listdir():
            try:
                i = int(filename.split(b'.')[0])
            except ValueError:
                continue
            if i > cur_high:
                cur_high = i
        return cur_high + 1

    def read_next_free():
        """Return next int free by reading the next_free file, or None"""
        rp = get_long_rp(counter_filename)
        if not rp.lstat():
            return None
        return int(rp.get_string())

    def write_next_free(i):
        """Write value i into the counter file"""
        rp = get_long_rp(counter_filename)
        if rp.lstat():
            rp.delete()
        rp.write_string(str(free_name_counter))
        rp.fsync_with_dir()

    if not free_name_counter:
        free_name_counter = read_next_free()
    if not free_name_counter:
        free_name_counter = scan_next_free()
    filename = b'%i' % free_name_counter
    rp = get_long_rp(filename)
    assert not rp.lstat(), "Unexpected file at '%s' found" % rp.get_safepath()
    free_name_counter += 1
    write_next_free(free_name_counter)
    return filename


def check_new_index(base, index, make_dirs=0):
    """Return new rpath with given index, or None if that is too long

    If make_dir is True, make any parent directories to assure that
    file is really too long, and not just in directories that don't exist.

    """

    def wrap_call(func, *args):
        try:
            result = func(*args)
        except EnvironmentError as exc:
            if (exc.errno == errno.ENAMETOOLONG):
                return None
            raise
        return result

    def make_parent(rp):
        parent = rp.get_parent_rp()
        if parent.lstat():
            return 1
        parent.makedirs()
        return 2

    rp = wrap_call(base.new_index, index)
    if not make_dirs or not rp or rp.lstat():
        return rp

    parent_result = wrap_call(make_parent, rp)
    if not parent_result:
        return None
    elif parent_result == 1:
        return rp
    else:
        return wrap_call(base.new_index, index)


def get_mirror_rp(mirror_base, mirror_rorp):
    """Get the mirror_rp for reading a regular file

    This will just be in the mirror_base, unless rorp has an alt
    mirror name specified.  Use new_rorp, unless it is None or empty,
    and mirror_rorp exists.

    """
    if mirror_rorp.has_alt_mirror_name():
        return get_long_rp(mirror_rorp.get_alt_mirror_name())
    else:
        rp = check_new_index(mirror_base, mirror_rorp.index)
        if rp:
            return rp
        else:
            raise Exception("the following line doesn't make any sense but does it matter?")
            # FIXME index isn't defined anywhere, is mirror_rorp.index meant?
            # return mirror_base.new_index_empty(index)


def get_mirror_inc_rps(rorp_pair, mirror_root, inc_root=None):
    """Get (mirror_rp, inc_rp) pair, possibly making new longname base

    To test inc_rp, pad incbase with 50 random (non-quoted) characters
    and see if that raises an error.

    """
    if not inc_root:  # make fake inc_root if not available
        inc_root = mirror_root.append_path(b'rdiff-backup-data/increments')

    def mir_triple_old(old_rorp):
        """Return (mirror_rp, alt_mirror, alt_inc) from old_rorp"""
        if old_rorp.has_alt_mirror_name():
            alt_mirror = old_rorp.get_alt_mirror_name()
            return (get_long_rp(alt_mirror), alt_mirror, None)
        else:
            mirror_rp = mirror_root.new_index(old_rorp.index)
            if old_rorp.has_alt_inc_name():
                return (mirror_rp, None, old_rorp.get_alt_inc_name())
            else:
                return (mirror_rp, None, None)

    def mir_triple_new(new_rorp):
        """Return (mirror_rp, alt_mirror, None) from new_rorp"""
        mirror_rp = check_new_index(mirror_root, new_rorp.index)
        if mirror_rp:
            return (mirror_rp, None, None)
        alt_mirror = get_next_free()
        return (get_long_rp(alt_mirror), alt_mirror, None)

    def update_rorp(new_rorp, alt_mirror, alt_inc):
        """Update new_rorp with alternate mirror/inc information"""
        if not new_rorp or not new_rorp.lstat():
            return
        if alt_mirror:
            new_rorp.set_alt_mirror_name(alt_mirror)
        elif alt_inc:
            new_rorp.set_alt_inc_name(alt_inc)

    def find_inc_pair(index, mirror_rp, alt_mirror, alt_inc):
        """Return (alt_inc, inc_rp) pair"""
        if alt_mirror:
            return (None, mirror_rp)
        elif alt_inc:
            return (alt_inc, get_long_rp(alt_inc))
        elif not index:
            return (None, inc_root)

        trial_inc_index = index[:-1] + (index[-1] + (b'a' * 50), )
        if check_new_index(inc_root, trial_inc_index, make_dirs=1):
            return (None, inc_root.new_index(index))
        alt_inc = get_next_free()
        return (alt_inc, get_long_rp(alt_inc))

    (new_rorp, old_rorp) = rorp_pair
    if old_rorp and old_rorp.lstat():
        mirror_rp, alt_mirror, alt_inc = mir_triple_old(old_rorp)
        index = old_rorp.index
    else:
        assert new_rorp and new_rorp.lstat(), (old_rorp, new_rorp)
        mirror_rp, alt_mirror, alt_inc = mir_triple_new(new_rorp)
        index = new_rorp.index

    alt_inc, inc_rp = find_inc_pair(index, mirror_rp, alt_mirror, alt_inc)
    update_rorp(new_rorp, alt_mirror, alt_inc)
    return mirror_rp, inc_rp


# ------------------------------------------------------------------
# The following section is for restoring

# This holds a dictionary {incbase: inclist}.  The keys are increment
# bases like '1' or '23', and the values are lists containing the
# associated increments.
restore_inc_cache = None


def set_restore_cache():
    """Initialize restore_inc_cache based on long filename dir"""
    global restore_inc_cache
    restore_inc_cache = {}
    root_rf = restore.RestoreFile(get_long_rp(), get_long_rp(), [])
    for incbase_rp, inclist in root_rf.yield_inc_complexes(get_long_rp()):
        restore_inc_cache[incbase_rp.index[-1]] = inclist


def get_inclist(inc_base_name):
    if not restore_inc_cache:
        set_restore_cache()
    try:
        return restore_inc_cache[inc_base_name]
    except KeyError:
        return []


def update_rf(rf, rorp, mirror_root):
    """Return new or updated restorefile based on alt name info in rorp"""

    def _safe_str(cmd):
        """Transform bytes into string without risk of conversion error"""
        if isinstance(cmd, str):
            return cmd
        else:
            return str(cmd, errors='replace')

    def update_incs(rf, inc_base):
        """Swap inclist in rf with those with base inc_base and return"""
        log.Log(
            "Restoring with increment base %s for file %s" %
            (_safe_str(inc_base), rorp.get_safeindexpath()), 6)
        rf.inc_rp = get_long_rp(inc_base)
        rf.inc_list = get_inclist(inc_base)
        rf.set_relevant_incs()

    def update_existing_rf(rf, rorp):
        """Update rf based on rorp, don't make new one"""
        if rorp.has_alt_mirror_name():
            inc_name = rorp.get_alt_mirror_name()
            raise Exception("the following line doesn't make any sense but does it matter?")
            # FIXME mirror_name isn't defined anywhere, is inc_name meant?
            # rf.mirror_rp = get_long_rp(mirror_name)
        elif rorp.has_alt_inc_name():
            inc_name = rorp.get_alt_inc_name()
        else:
            inc_name = None

        if inc_name:
            update_incs(rf, inc_name)

    def make_new_rf(rorp, mirror_root):
        """Make a new rf when long name info is available"""
        if rorp.has_alt_mirror_name():
            inc_name = rorp.get_alt_mirror_name()
            mirror_rp = get_long_rp(inc_name)
        else:
            mirror_rp = mirror_root.new_index(rorp.index)
            if rorp.has_alt_inc_name():
                inc_name = rorp.get_alt_inc_name()
            else:
                return restore.RestoreFile(mirror_rp, None, [])

        rf = restore.RestoreFile(mirror_rp, None, [])
        update_incs(rf, inc_name)
        return rf

    if not rorp:
        return rf
    if rf and not rorp.has_alt_mirror_name() and not rorp.has_alt_inc_name():
        return rf  # Most common case
    if rf:
        update_existing_rf(rf, rorp)
        return rf
    else:
        return make_new_rf(rorp, mirror_root)


def update_regressfile(rf, rorp, mirror_root):
    """Like update_rf except return a regress file object"""
    rf = update_rf(rf, rorp, mirror_root)
    if isinstance(rf, regress.RegressFile):
        return rf
    return regress.RegressFile(rf.mirror_rp, rf.inc_rp, rf.inc_list)



import os
import unittest
from rdiff_backup import robust


class RobustTest(unittest.TestCase):
    """Test robust module"""

    def test_check_common_error(self):
        """Test capturing errors"""

        def cause_catchable_error(a):
            os.lstat("aoenuthaoeu/aosutnhcg.4fpr,38p")

        def cause_uncatchable_error():
            ansoethusaotneuhsaotneuhsaontehuaou  # noqa: F821 undefined name

        result = robust.check_common_error(None, cause_catchable_error, [1])
        assert result is None, result
        try:
            robust.check_common_error(None, cause_uncatchable_error)
        except NameError:
            pass
        else:
            assert 0, "Key error not raised"


if __name__ == '__main__':
    unittest.main()



import unittest
import pwd
import code
from rdiff_backup import user_group, Globals


class UserGroupTest(unittest.TestCase):
    """Test user and group functionality"""

    def test_basic_conversion(self):
        """Test basic id2name.  May need to modify for different systems"""
        user_group.uid2uname_dict = {}
        user_group.gid2gname_dict = {}
        assert user_group.uid2uname(0) == "root"
        assert user_group.uid2uname(0) == "root"
        assert user_group.gid2gname(0) == "root"
        assert user_group.gid2gname(0) == "root"
        # Assume no user has uid 29378
        assert user_group.gid2gname(29378) is None
        assert user_group.gid2gname(29378) is None

    def test_basic_reverse(self):
        """Test basic name2id.  Depends on systems users/groups"""
        user_group.uname2uid_dict = {}
        user_group.gname2gid_dict = {}
        assert user_group.uname2uid("root") == 0
        assert user_group.uname2uid("root") == 0
        assert user_group.gname2gid("root") == 0
        assert user_group.gname2gid("root") == 0
        assert user_group.uname2uid("aoeuth3t2ug89") is None
        assert user_group.uname2uid("aoeuth3t2ug89") is None

    def test_default_mapping(self):
        """Test the default user mapping"""
        Globals.isdest = 1
        rootid = 0
        binid = pwd.getpwnam('bin')[2]
        syncid = pwd.getpwnam('sync')[2]
        user_group.init_user_mapping()
        assert user_group.UserMap(0) == rootid
        assert user_group.UserMap(0, 'bin') == binid
        assert user_group.UserMap(0, 'sync') == syncid
        assert user_group.UserMap.map_acl(0, 'aoeuth3t2ug89') is None

    def test_user_mapping(self):
        """Test the user mapping file through the DefinedMap class"""
        mapping_string = """
root:bin
bin:root
500:501
0:sync
sync:0"""
        Globals.isdest = 1
        rootid = 0
        binid = pwd.getpwnam('bin')[2]
        syncid = pwd.getpwnam('sync')[2]
        daemonid = pwd.getpwnam('daemon')[2]
        user_group.init_user_mapping(mapping_string)

        assert user_group.UserMap(rootid, 'root') == binid
        assert user_group.UserMap(binid, 'bin') == rootid
        assert user_group.UserMap(0) == syncid
        assert user_group.UserMap(syncid, 'sync') == 0
        assert user_group.UserMap(500) == 501

        assert user_group.UserMap(501) == 501
        assert user_group.UserMap(123, 'daemon') == daemonid

        assert user_group.UserMap.map_acl(29378, 'aoeuth3t2ug89') is None
        assert user_group.UserMap.map_acl(0, 'aoeuth3t2ug89') is syncid

        if 0:
            code.InteractiveConsole(globals()).interact()

    def test_overflow(self):
        """Make sure querying large uids/gids doesn't raise exception"""
        large_num = 4000000000
        assert user_group.uid2uname(large_num) is None
        assert user_group.gid2gname(large_num) is None


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import time
from commontest import abs_test_dir, Myrm
from rdiff_backup import Globals, rpath, fs_abilities


class FSAbilitiesTest(unittest.TestCase):
    """Test testing of file system abilities

    Some of these tests assume that the actual file system tested has
    the given abilities.  If the file system this is run on differs
    from the original test system, this test may/should fail. Change
    the expected values below.

    """
    # Describes standard linux file system without acls/eas
    dir_to_test = abs_test_dir
    eas = acls = 1
    chars_to_quote = ""
    extended_filenames = 1
    case_sensitive = 1
    ownership = (os.getuid() == 0)
    hardlinks = fsync_dirs = 1
    dir_inc_perms = 1
    resource_forks = 0
    carbonfile = 0
    high_perms = 1

    # Describes MS-Windows style file system
    # dir_to_test = "/mnt/fat"
    # eas = acls = 0
    # extended_filenames = 0
    # chars_to_quote = "^a-z0-9_ -"
    # ownership = hardlinks = 0
    # fsync_dirs = 1
    # dir_inc_perms = 0
    # resource_forks = 0
    # carbonfile = 0

    # A case insensitive directory (FIXME must currently be created by root)
    # mkdir build/testfiles/fs_insensitive
    # dd if=/dev/zero of=build/testfiles/fs_fatfile.dd bs=512 count=1024
    # mkfs.fat build/testfiles/fs_fatfile.dd
    # sudo mount -o loop,uid=$(id -u) build/testfiles/fs_fatfile.dd build/testfiles/fs_insensitive
    # touch build/testfiles/fs_fatfile.dd build/testfiles/fs_insensitive/some_File

    case_insensitive_path = os.path.join(abs_test_dir, b'fs_insensitive')

    def testReadOnly(self):
        """Test basic querying read only"""
        base_dir = rpath.RPath(Globals.local_connection, self.dir_to_test)
        fsa = fs_abilities.FSAbilities('read-only').init_readonly(base_dir)
        print(fsa)
        assert fsa.read_only == 1, fsa.read_only
        assert fsa.eas == self.eas, fsa.eas
        assert fsa.acls == self.acls, fsa.acls
        assert fsa.resource_forks == self.resource_forks, fsa.resource_forks
        assert fsa.carbonfile == self.carbonfile, fsa.carbonfile
        assert fsa.case_sensitive == self.case_sensitive, fsa.case_sensitive

    def testReadWrite(self):
        """Test basic querying read/write"""
        base_dir = rpath.RPath(Globals.local_connection, self.dir_to_test)
        new_dir = base_dir.append("fs_abilitiestest")
        if new_dir.lstat():
            Myrm(new_dir.path)
        new_dir.setdata()
        new_dir.mkdir()
        t = time.time()
        fsa = fs_abilities.FSAbilities('read/write').init_readwrite(new_dir)
        print("Time elapsed = ", time.time() - t)
        print(fsa)
        assert fsa.read_only == 0, fsa.read_only
        assert fsa.eas == self.eas, fsa.eas
        assert fsa.acls == self.acls, fsa.acls
        assert fsa.ownership == self.ownership, fsa.ownership
        assert fsa.hardlinks == self.hardlinks, fsa.hardlinks
        assert fsa.fsync_dirs == self.fsync_dirs, fsa.fsync_dirs
        assert fsa.dir_inc_perms == self.dir_inc_perms, fsa.dir_inc_perms
        assert fsa.resource_forks == self.resource_forks, fsa.resource_forks
        assert fsa.carbonfile == self.carbonfile, fsa.carbonfile
        assert fsa.high_perms == self.high_perms, fsa.high_perms
        assert fsa.extended_filenames == self.extended_filenames

        new_dir.delete()

    @unittest.skipUnless(os.path.isdir(case_insensitive_path),
                         "Case insensitive directory %s does not exist" %
                         case_insensitive_path)
    def test_case_sensitive(self):
        """Test a read-only case-INsensitive directory"""
        rp = rpath.RPath(Globals.local_connection, self.case_insensitive_path)
        fsa = fs_abilities.FSAbilities('read-only')
        fsa.set_case_sensitive_readonly(rp)
        assert fsa.case_sensitive == 0, fsa.case_sensitive


if __name__ == "__main__":
    unittest.main()



import unittest
import os
from commontest import MakeOutputDir, abs_output_dir, abs_test_dir, old_test_dir, \
    re_init_rpath_dir, rdiff_backup
from rdiff_backup import FilenameMapping, rpath, Globals


class FilenameMappingTest(unittest.TestCase):
    """Test the FilenameMapping class, for quoting filenames"""

    def setUp(self):
        """Just initialize quoting"""
        Globals.chars_to_quote = b'A-Z'
        FilenameMapping.set_init_quote_vals()

    def testBasicQuote(self):
        """Test basic quoting and unquoting"""
        filenames = [
            b"hello", b"HeLLo", b"EUOeu/EUOeu", b":", b"::::EU", b"/:/:"
        ]
        for filename in filenames:
            quoted = FilenameMapping.quote(filename)
            assert FilenameMapping.unquote(quoted) == filename, filename

    def testQuotedRPath(self):
        """Test the QuotedRPath class"""

    def testQuotedSepBase(self):
        """Test get_quoted_sep_base function"""
        path = (b"/usr/local/mirror_metadata"
                b".1969-12-31;08421;05833;05820-07;05800.data.gz")
        qrp = FilenameMapping.get_quoted_sep_base(path)
        assert qrp.base == b"/usr/local", qrp.base
        assert len(qrp.index) == 1, qrp.index
        assert (qrp.index[0] == b"mirror_metadata.1969-12-31T21:33:20-07:00.data.gz")

    def testLongFilenames(self):
        """See if long quoted filenames cause crash"""
        MakeOutputDir()
        outrp = rpath.RPath(Globals.local_connection, abs_output_dir)
        inrp = rpath.RPath(Globals.local_connection,
                           os.path.join(abs_test_dir, b"quotetest"))
        re_init_rpath_dir(inrp)
        long_filename = b"A" * 200  # when quoted should cause overflow
        longrp = inrp.append(long_filename)
        longrp.touch()
        shortrp = inrp.append(b"B")
        shortrp.touch()

        rdiff_backup(1,
                     1,
                     inrp.path,
                     outrp.path,
                     100000,
                     extra_options=b"--override-chars-to-quote A")

        longrp_out = outrp.append(long_filename)
        assert not longrp_out.lstat()
        shortrp_out = outrp.append('B')
        assert shortrp_out.lstat()

        rdiff_backup(1, 1, os.path.join(old_test_dir, b"empty"), outrp.path,
                     200000)
        shortrp_out.setdata()
        assert not shortrp_out.lstat()
        rdiff_backup(1, 1, inrp.path, outrp.path, 300000)
        shortrp_out.setdata()
        assert shortrp_out.lstat()


if __name__ == "__main__":
    unittest.main()
